/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX Sistemi Integrati. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2007. 
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
/* ===================================================================== */


#include <Foundation/logging/LogLevel.hpp>
#include <Foundation/osthreads/MutexGuard.hpp>
#include <Foundation/logging/LogManager.hpp>

#include "logging/LogManagerCollectorDelegate.hpp"


namespace Cdmw
{
    namespace Logging
    {
        /**
         * Constructor.
         */
        LogManagerCollectorDelegate::LogManagerCollectorDelegate(CdmwLogging::LogCollector_ptr ref_collector)
            throw(OutOfMemoryException)
        {
           CDMW_ASSERT(!CORBA::is_nil(ref_collector));
           m_ref_collector = CdmwLogging::LogCollector::_duplicate(ref_collector);
        }

        /**
         * Destructor.
         */
        LogManagerCollectorDelegate::~LogManagerCollectorDelegate()
            throw()
        {
        }


        /**
         * Send a message to the logger.
         */
        bool
        LogManagerCollectorDelegate::add_log(const CdmwLogging::LogContext& context,
                                             const CdmwLogging::LogHeader& header,
                                             const char* msg)
            throw(InternalErrorException)
        {
            bool result = true;
            try
            {
                CdmwLogging::StructuredLog log;
                log.ctx = context;
                log.header = header;
                log.message = msg;
                result = m_ref_collector->add_log(log);
            }
            catch (const ::CORBA::SystemException& se)
            {
                std::cerr << "Cannot log message, unexpected CORBA::SystemException raised: "
                          << se << " (minor = " << se.minor() << ")" << std::endl;
                result = false;
            }
    
            return result;
        }

    } // end namespace Logging
} // end namespace Cdmw
