/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX Sistemi Integrati. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2007. 
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
/* ===================================================================== */

package cdmw.logging;

import java.io.*;
import java.lang.Long;
import cdmw.ossupport.*;
import cdmw.common.*;

import com.thalesgroup.CdmwLogging.LogHeader;
import com.thalesgroup.CdmwLogging.LogContext;
import com.thalesgroup.CdmwLogging.StructuredLog;

import java.security.InvalidAlgorithmParameterException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.NoSuchElementException;
import java.util.StringTokenizer;
import java.lang.String;

/**
 * This class enables logging in standard output or file.
 */
public class LogManagerDefaultDelegate implements LogManagerDelegate
{

    /**
     * logs output; if null, standard output is used.
     */ 
    private BufferedWriter ofstream;


    /**
     * Constructor.
     */
    public LogManagerDefaultDelegate() {
        this.ofstream = null;
    }

    /**
     * Constructor.
     * @param fileName output log file
     * @exception BadParameterException when the specified file is not writable
     */
    public LogManagerDefaultDelegate(String fileName) throws BadParameterException {
        try {
            this.ofstream = new BufferedWriter(new FileWriter(fileName, true));
            System.out.println("CDMW Log set to " + fileName);
        }
        catch (IOException ioe) {
            throw new BadParameterException("Couldn't create log file " + fileName + ": "
                    + ioe.getMessage());
        }
    }

    
    // TODO: This should be moved to OsSupport
    String basename(String f) {
        String DELIM= "/";
        int i = f.lastIndexOf(DELIM);
        if (i != -1) 
        {
            String res = f.substring(i+1,f.length());
            return res;
        } 
        else
        {
            return f;
        }

    } 

    /**
     * Converts a LogHeader into a formatted string.
     * 
     * @param header log informations
     * @return the build string 
     */
    protected String LogHeaderToString(LogHeader header) {

        // write time with second precision in buffer
        SimpleDateFormat tf = new SimpleDateFormat("yyyy/MM/dd HH:mm:ss:SSS");
        GregorianCalendar calendar = new GregorianCalendar();
        calendar.setTime(
                new Date((long) 
                header.time_stamping.seconds * 1000
                + header.time_stamping.microseconds / 1000)
                );
        String buffer1 = tf.format(calendar.getTime());
        // add milliseconds and microseconds in buffer

        String buffer3 =  ":000 " + LogLevel.LevelToString(header.level)
            + " [" + header.component_name + "]" + " "
            + header.user_domain + " [tid =" + header.thread_id + "] ";

        String buffer3a =  "(" + basename(header.file_name) + ":" + header.file_line + ")";

        return buffer1 + buffer3 + buffer3a;        
    }

    /**
     * Send a message to the logger.
     *
     * @param context the log context.
     * @param header the message header.
     * @param msg the message.
     *
     * @return true if successful.
     */
    public boolean addLog(LogContext context, LogHeader header, String msg) {
        
        boolean result = true;
        try
        {
            // context not logged
            String buffer = LogHeaderToString(header);
            buffer += " [" + context.ndc + "] " + '\t' + msg;

            synchronized (this) {
                if (this.ofstream != null) {
                    this.ofstream.write(buffer);
                    this.ofstream.newLine();
                    this.ofstream.flush();
                }
                else {
                    System.out.println(buffer);
                }
            }
        }
        catch (IOException e) {
            System.out.println("Cannot log message: " + e.getMessage());
            result = false;
        }

        return result;
    }


    @Override
    protected void finalize() throws Throwable {
        super.finalize();
        this.ofstream.close();
    }

}
