/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX Sistemi Integrati. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2007. 
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <iostream>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/logging/LogManager.hpp>

#include "ccm_and_lb/ClientExecutor_impl.hpp"



namespace CcmAndLB
{

/**
 * IDL:thalesgroup.com/CcmAndLB/CCM_Client:1.0
 */


    /**
     * Constructor.
     */
    ClientExecutor_impl::ClientExecutor_impl()
        : m_session_context(CCM_Client_Context::_nil()),
          m_logger(Cdmw::Logging::Logger::Get_Logger("CCM_Client",
                                                     "QUALIF_CCM_AND_LB"))
    {
    }


    /**
     * Destructor.
     */
    ClientExecutor_impl::~ClientExecutor_impl()
    {
        CDMW_INFO(m_logger, "ClientExecutor_impl::~ClientExecutor_impl() called.");
    }


    /**
     * Set the session context of the component (called by the container).
     */
    void
    ClientExecutor_impl::set_session_context(
        Components::SessionContext_ptr ctx
        )
        throw(CORBA::SystemException)
    {
        CDMW_INFO(m_logger,
                  "ClientExecutor_impl::set_session_context() called.");

        m_session_context = CCM_Client_Context::_narrow(ctx);
    }


    /**
     * Activate the component (called by the container).
     */
    void
    ClientExecutor_impl::ccm_activate()
        throw(CORBA::SystemException)
    {
        CDMW_INFO(m_logger, "ClientExecutor_impl::ccm_activate() called.");

        int numOfLBGroupConnections = 100;   
        int numOfClientServerConnections = 5;
        CORBA::String_var process_name = CORBA::string_dup("");
        try {
            // get the display connection of the server.
            CDMW_INFO(m_logger, "connect to the display facet...");

            for (int ji = 0 ; ji<numOfLBGroupConnections; ji++) 
            {
               std::cerr << " *** Component Client (c++) connection n. " << ji+1 << " to LB Compoments Group.. " << std::endl; 
               HelloWorld_var display =
                   m_session_context->get_connection_client_receptacle();

               // call the server display.
               for (int i = 0 ; i<numOfClientServerConnections; i++) 
               {
               
                  std::cerr << " *** Component Client (c++) call sayHello " << std::endl; 
                  //Cdmw::OsSupport::OS::sleep(1000);
                  display->sayHello();
                  
               }
               CORBA::String_var out_;
               CORBA::String_var ret_;
               //ret_ = display->printName(out_.out());
            }


        }
        catch (const CORBA::Exception& e) {
            CDMW_ERROR(m_logger, "unexpected exception raised: " << e);
        }
        catch (...) {
            CDMW_ERROR(m_logger, "unexpected exception raised.");
        }
    }


    /**
     * Stop the component (called by the container).
     */
    void
    ClientExecutor_impl::ccm_passivate()
        throw(CORBA::SystemException)
    {
        CDMW_INFO(m_logger, "ClientExecutor_impl::ccm_passivate() called.");
    }


    /**
     * Destroy the component (called by the container).
     */
    void
    ClientExecutor_impl::ccm_remove()
        throw(CORBA::SystemException)
    {
        CDMW_INFO(m_logger, "ClientExecutor_impl::ccm_remove() called.");
    }



  void ClientExecutor_impl::cdmw_initialise_component(const ::CdmwPlatformMngtBase::StartupKind& startup_kind)
      throw(CdmwPlatformMngt::ManagedComponent::BadOrder, CORBA::SystemException)
  {
     std::cerr << " *** Component Client initialisation ****  " << std::endl;
        
     Cdmw::OsSupport::OS::sleep(5000);
     std::cerr << " *** Component Client initialised ****  " << std::endl;
  }
  void ClientExecutor_impl::handle_set(const CORBA::Any& params)
    throw (CORBA::SystemException) {
  }

  void ClientExecutor_impl::handle_set_rate(const CORBA::Any& params)
    throw (CORBA::SystemException) {
  }


  void ClientExecutor_impl::handle_pause(const CORBA::Any& params)
    throw (CORBA::SystemException) {
  }

  void ClientExecutor_impl::handle_resume(const CORBA::Any& params)
    throw (CORBA::SystemException) {
  }

  void ClientExecutor_impl::handle_terminate(const CORBA::Any& params)
    throw (CORBA::SystemException) {
  }
 

}; // End of namespace CcmAndLB
