/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX Sistemi Integrati. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2007. 
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
/* ===================================================================== */



#include <ccm_ft_events/CCM_Server_impl.hpp>

#include <Foundation/common/Assert.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/osthreads/Mutex.hpp>
#include "ccm_ft_events/CCM_HostInfo_impl.hpp"
#include "ccm_ft_events/ccm_ft_events_cif.skel.hpp"
#include "ccm_ft_events/EventTest_impl.hpp"
#include "ccm_ft_events/SecondEventTest_impl.hpp"

// static variables
Cdmw::OsSupport::Mutex CcmFtTest::CCM_Server_impl::M_mutex;
bool CcmFtTest::CCM_Server_impl::M_ack_evts = false;
Cdmw::OsSupport::Condition CcmFtTest::CCM_Server_impl::M_condition(CcmFtTest::CCM_Server_impl::M_mutex);
std::string CcmFtTest::CCM_Server_impl::M_host1;
std::string CcmFtTest::CCM_Server_impl::M_host2;
struct CcmFtTest::EventInfos CcmFtTest::CCM_Server_impl::M_event_infos;

namespace {
  
  // Send the events and wait that all the events have been received
  void send_and_wait (CcmFtTest::CCM_Server_Context& ctx, void (CcmFtTest::CCM_Server_Context::*send_event)(CcmFtTest::EventTest *), void (CcmFtTest::CCM_Server_Context::*send_s_event)(CcmFtTest::SecondEventTest *))
  {
    ::CcmFtTest::EventTest_var evt;
    ::CcmFtTest::SecondEventTest_var s_evt;

    if (send_event != NULL)
      evt = new ::CcmFtTest::EventTest_impl();
    else
      s_evt = new ::CcmFtTest::SecondEventTest_impl();

    int i;
    int event_id;

    // Send the events
    for ( i = 0; i < MAX_NB_EVENTS; i++ ) {

      // At the same time, initialization of m_event_infos
      CcmFtTest::CCM_Server_impl::M_event_infos.events [ i ] = false;
      CcmFtTest::CCM_Server_impl::M_event_infos.hostname [ i ] = "";

      event_id = i + 1;

      std::cout << "Server: Sending of the event #" << event_id <<  std::endl;

      if (send_event != NULL){
    evt -> numOfEvent ( event_id );
    (ctx.*send_event) ( evt.in() );
      }
      else {
    s_evt -> numOfEvent ( event_id );
    (ctx.*send_s_event) ( s_evt.in() );
      }

    }
    
    typedef CcmFtTest::CCM_Server_impl CCM_Server_impl;

    // Before continuing, wait that all the calls to HostInfo facets end up
    {
      CDMW_MUTEX_GUARD(CCM_Server_impl::M_mutex);
      try {
    while (!CCM_Server_impl::M_ack_evts){
          if ( CCM_Server_impl::M_condition.wait(20 * 1000)
               == Cdmw::OsSupport::Condition::TIMEDOUT )
            {
              std::cout << "\nWarning: Timeout expired. 20 secs were not enough to receive all events from client component!"
                << std::endl;
              CPPUNIT_ASSERT(false);
            }
    }
            } catch (const Cdmw::InternalErrorException& ex) {
          CPPUNIT_ASSERT(false);
          CDMW_ASSERT(false);
            }
    }
    
    CCM_Server_impl::M_ack_evts = false;

    std::cout << "\nCheck that all events were received..."
          << std::endl;

  }
}

//
// IDL:thalesgroup.com/CcmFtTest/CCM_Server:1.0
//
CcmFtTest::CCM_Server_impl::CCM_Server_impl()
    : m_session_context(CcmFtTest::CCM_Server_Context::_nil()),
      m_testThread()
{
  m_host_info = new ::CcmFtTest::CCM_HostInfo_impl();
}

CcmFtTest::CCM_Server_impl::~CCM_Server_impl()
{
    std::cout << "   ... CCM_Server_impl::~CCM_Server_impl called!" << std::endl;
}

//
// set_session_context function : it fixes the session context of the component (call by the container)
//
void
CcmFtTest::CCM_Server_impl::set_session_context(Components::SessionContext_ptr ctx)
    throw(CORBA::SystemException)
{
    std::cout << "   ... CCM_Server_impl::Set_session_context called!" << std::endl;
   
    m_session_context = ::CcmFtTest::CCM_Server_Context::_narrow(ctx); 
}


//
// ccm_activate function : it actives the component (call by the container)
//
void
CcmFtTest::CCM_Server_impl::ccm_activate()
   throw(CORBA::SystemException)
{
   std::cout << "   ... CCM_Server_impl::Ccm_activate called!" << std::endl;
   Cdmw::TestUtils::CORBATestManager::instance()->add("server_context", get_context());
   Cdmw::TestUtils::CORBATestManager::instance()->add("ccm_server", static_cast< ::CcmFtTest::CCM_Server_ptr > (this));
   m_testThread.start();
}


//
// ccm_passivate function : it makes stop the component (call by the container)
//
void
CcmFtTest::CCM_Server_impl::ccm_passivate()
   throw(CORBA::SystemException)
{
   std::cout << "   ... CCM_Server_impl::Ccm_passivate called!" << std::endl;
} 



//
// ccm_remove function : it destroys the component (call by the container)
//
void
CcmFtTest::CCM_Server_impl::ccm_remove()
   throw(CORBA::SystemException)
{
   std::cout << "   ... CCM_Server_impl::Ccm_remove called!" << std::endl;
}

//
CPPUNIT_TEST_SUITE_NAMED_REGISTRATION(CcmFtTest::test_ccm_ft_activate, "test_ccm_ft_activate");

void
CcmFtTest::test_ccm_ft_activate::do_tests()
{
    
    std::cout << "Starting of the test" << std::endl;

    CORBA::Object_var obj2 = Cdmw::TestUtils::CORBATestManager::instance()->get_object("ccm_server");
    ::CcmFtTest::CCM_Server_var ccm_server = ::CcmFtTest::CCM_Server::_narrow(obj2.in());
    m_ccm_server = reinterpret_cast< CcmFtTest::CCM_Server_impl* > (ccm_server.in());

    m_session_context = m_ccm_server->get_context();

    ::send_and_wait(*m_session_context, &CcmFtTest::CCM_Server_Context::push_eventEmits, NULL);
 
    int event_id;
    int i;
    bool check_ok = true;
    std::string hostname;

    for ( i = 0; i < MAX_NB_EVENTS; i++ ) {
      
      event_id = i + 1;
      hostname = CCM_Server_impl::M_event_infos.hostname[i];

      if ( CCM_Server_impl::M_event_infos.events [ i ] == true ) {
    if (
        (i < MAX_NB_EVENTS / 2 
         && hostname.compare(CCM_Server_impl::M_host1) == 0) 
        ||
        (i >= MAX_NB_EVENTS / 2 - 1
         && hostname.compare(CCM_Server_impl::M_host2) == 0)
        ) {

      std::cout << "Server: event #" << event_id << " received by "
            << hostname
            << std::endl;

      CPPUNIT_ASSERT(check_ok);
    }
    else {
      std::cout << "Error1 - Server: event #" << event_id << " received by "
            << hostname
            << std::endl;

      check_ok = false;
      break;
    }
      }
      else {
    std::cout << "Error2 - Server: event #" << event_id << " not received !"
          << std::endl;
    check_ok = false;
    break;
      }
    }

    if ( check_ok )
      std::cout << "\nAll events were received\n"<< std::endl;
    else {
      std::cout << "\nAll events were not received!\n"<< std::endl;
      CPPUNIT_ASSERT(check_ok);
    }

    ::send_and_wait(*m_session_context, NULL, &CcmFtTest::CCM_Server_Context:: push_secondEventEmits);

    check_ok = true;
    for ( i = 0; i < MAX_NB_EVENTS; i++ ) {
      
      event_id = i + 1;
      hostname = CCM_Server_impl::M_event_infos.hostname[i];

      if ( CCM_Server_impl::M_event_infos.events [ i ] == true ) {
    std::cout << "Server: event #" << event_id << " received by "
          << hostname
          << std::endl;

      CPPUNIT_ASSERT(check_ok);
    }
      else {
    std::cout << "Error - Server: event #" << event_id << " received by "
          << hostname
          << std::endl;
    
    check_ok = false;
    break;
      }
    }

    if ( check_ok ){
      std::cout << "\n===================================================" << std::endl;
      std::cout << "Ending test Test ccm_activate on Server" << std::endl;
      std::cout << "===================================================" << std::endl;
      std::cout << "\nAll events were received. The test is finished...\n"<< std::endl;
    }
    else {
      std::cout << "\nAll events were not received!\n"<< std::endl;
    }

    CPPUNIT_ASSERT(check_ok);

}


CcmFtTest::CCM_HostInfo_ptr
CcmFtTest::CCM_Server_impl::get_serverHostInfo ( )
  throw ( CORBA::SystemException )
{
  return  CcmFtTest::CCM_HostInfo::_duplicate(m_host_info.in());
}


