/* ========================================================================== *
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI.
 * Copyright (C) SELEX SISTEMI INTEGRATI 2004-2007. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * ========================================================================= */
 
#include "daemoncontroller/DaemonController_impl.hpp"

const std::string FILE_LOG = "tmp/daemoncontrollerlog.txt";
//write in log file
extern void writeRowInLog(std::string value);
extern std::string BASE_EXIT_ACTION;
namespace Cdmw
{
    std::string m_stop_host_script = "cdmw_local_host_stop.sh";    
    std::string m_start_host_script = "cdmw_host_start.sh";
    std::string m_reboot_host_script = "cdmw_host_reboot.sh";
    std::string m_shutdown_host_script = "cdmw_host_shutdown.sh";
    bool oldNotification = false;
    
    DaemonController_impl::DaemonController_impl()
        throw(CORBA::SystemException)
    {
    }
    
    DaemonController_impl::~DaemonController_impl()
        throw()
    {
    }
    
    void execute_shell_command(std::string command)
    throw (CORBA::NO_PERMISSION)
    {
        try
        {
             OsSupport::OS::create_process("/bin/sh" , command , "", NULL, NULL);
        }
        catch (...)
        {
             throw CORBA::NO_PERMISSION(OrbSupport::NO_PERMISSION, CORBA::COMPLETED_NO);
        }
    }
    void DaemonController_impl::notify (short exitcode) throw(CORBA::SystemException)
    {
       std::cout << "EXIT ACTION: " << exitcode << std::endl;
       writeRowInLog("EXIT ACTION"); 
       switch (exitcode){
           case(0):
                if(!oldNotification){
                    writeRowInLog("Daemon dies");
                    if(BASE_EXIT_ACTION=="restart"){
                    //For restart
                        OsSupport::OS::set_process_attributes(0, 0);
                        OsSupport::OS::create_process(m_start_host_script, "", "", NULL, NULL);
                    }
                    else if(BASE_EXIT_ACTION=="reboot"){
                        OsSupport::OS::create_process(m_reboot_host_script , "" , "", NULL, NULL);
                    }         
                    else if(BASE_EXIT_ACTION=="exit_only"){
                        OsSupport::OS::create_process(m_stop_host_script , "" , "", NULL, NULL);
                    }
                }
                else oldNotification = false;
            break;
            case(2):
            writeRowInLog("Stop request, stop host in progress");
                oldNotification = true;
                OsSupport::OS::create_process(m_stop_host_script , "" , "", NULL, NULL);
            break;
            case(3):
            writeRowInLog("Reboot request, reboot in progress");
                oldNotification = true;
                OsSupport::OS::create_process(m_reboot_host_script , "" , "", NULL, NULL);
            break;
            case(4):
                //For shutdown
                writeRowInLog("Shutdown request , shutdown in progress");
                oldNotification = true;
                OsSupport::OS::set_process_attributes(0, 0);
                OsSupport::OS::create_process(m_shutdown_host_script, "", "", NULL, NULL);
                break;
            case(5):
                //For restart
                writeRowInLog("Restart request, restart in progress");
                oldNotification = true;
                OsSupport::OS::set_process_attributes(0, 0);
                OsSupport::OS::create_process(m_stop_host_script , "" , "", NULL, NULL);
                sleep(1);
                OsSupport::OS::create_process(m_start_host_script, "", "", NULL, NULL);
                break;
            default:
                writeRowInLog("Request to monitor unknown ");
           } 
    }   
};
