
/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX Sistemi Integrati. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2007. 
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
/* ===================================================================== */

#include "SystemMngt/platformelementlibrary/EntityContainer_impl.hpp"
#include "SystemMngt/platformelementlibrary/RWEntityContainer_impl.hpp"
#include "SystemMngt/platformelementlibrary/EntityContainer.hpp"

#include "Foundation/common/Exception.hpp"
#include "Foundation/common/Assert.hpp"
#include "Foundation/orbsupport/StrategyList.hpp"
#include "SystemMngt/idllib/CdmwPlatformMngtBase.stub.hpp"
#include "SystemMngt/idllib/CdmwPlatformMngtComponent.skel.hpp"
#include "platformapplicationlibrary/CcmComponent_impl.hpp"
#include "SystemMngt/platforminterface/PlatformInterface.hpp"
#include "SystemMngt/platforminterface/ServiceNames.hpp"
#include "Foundation/orbsupport/ExceptionMinorCodes.hpp"
#include "SystemMngt/platformlibrary/Configuration.hpp"
#include "SystemMngt/platformlibrary/EventHeaderFactory.hpp"
#include "SystemMngt/platformlibrary/EventNotifierManager.hpp"
#include "SystemMngt/platformlibrary/SupervisionEventNotifier.hpp"

#include "idllib/CdmwPlatformMngtSupervisionObserver.skel.hpp"
#include "Repository/repositoryinterface/RepositoryInterface.hpp"
#include "idllib/CdmwPlatformMngtPredefinedProperties.stub.hpp"

#include "SystemMngt/platformvaluetypes/ComponentStatusChange_impl.hpp"

#include "Foundation/logging/Logger.hpp"
#include <iostream>
#include <sstream>


namespace Cdmw
{
  namespace PlatformMngt
  {

       ComponentInitial* CcmComponent_impl::M_initial = 0;
       ComponentInitialising* CcmComponent_impl::M_initialising = 0;
       ComponentConfiguring* CcmComponent_impl::M_configuring = 0;
       ComponentInitialised* CcmComponent_impl::M_initialised = 0;
       ComponentConfigured* CcmComponent_impl::M_configured = 0;
       ComponentStopping* CcmComponent_impl::M_stopping = 0;
       ComponentStopped* CcmComponent_impl::M_stopped = 0;
       ComponentFailedError* CcmComponent_impl::M_failedError = 0;
       ComponentFailedTimeout* CcmComponent_impl::M_failedTimeout = 0;
       ComponentFailedInvalid* CcmComponent_impl::M_failedInvalid = 0;
       
       const string CcmComponent_impl::M_initialStateName                   = "Initial";
       const string CcmComponent_impl::M_initialisingStateName              = "Initialising";
       const string CcmComponent_impl::M_configuringStateName               = "Configuring";
       const string CcmComponent_impl::M_configuredStateName                = "Configured";
       const string CcmComponent_impl::M_initialisedStateName               = "Initialised";
       const string CcmComponent_impl::M_stoppingStateName                  = "Stopping";
       const string CcmComponent_impl::M_stoppedStateName                   = "Stopped";
       const string CcmComponent_impl::M_failedErrorStateName               = "FailedError";
       const string CcmComponent_impl::M_failedTimeoutStateName             = "FailedTimeout";
       const string CcmComponent_impl::M_failedInvalidStateName             = "FailedInvalid";

   
    CcmComponent_impl::CcmComponent_impl (PortableServer::POA_ptr poa,
                      const CdmwPlatformMngt::ComponentDef& ccmcomponent_def, 
                      CdmwPlatformMngt::RWElementRepository_ptr element_repository,
                      const char *component_key, const char *component_path,
                      Cdmw::PlatformMngt::Process_impl* process_impl):
                          ManagedElement_impl(element_repository, poa),
                          m_process_impl (process_impl)
    {
        CORBA::Object_var ccm_object = CORBA::Object::_nil();
        m_stateMachine = NULL;
        //m_entityStatusChangeFactory = 0;
        m_compStatusChangeFactory = 0;
        m_redundancy_group= 0;
        m_startup_kind = new CdmwPlatformMngtBase::StartupKind();
        
        //set element_path e name attributes values ...
        set_element_path (component_path, ccmcomponent_def.name );
        m_name =  ccmcomponent_def.name;
        m_deployment_plan = CORBA::string_dup (ccmcomponent_def.deployment_plan);
        m_redundancy_group_name = CORBA::string_dup (ccmcomponent_def.redundancy_group_name);
        m_redundancy_group_type = ccmcomponent_def.redundancy_group_type;
        m_statusInfo = CORBA::string_dup ("");
        
        // set in Managed Element the instance of Property Event Notifier Thread
        set_property_event_notifier(EventNotifierManager::Instance()->get_property_notifier());
        // set in Managed Element the instance of Status Event Notifier Thread
        set_status_event_notifier(EventNotifierManager::Instance()->get_status_notifier());
        
        //Adding the component in to the repective group
        if (CdmwPlatformMngt::COMPONENT_TYPE != m_redundancy_group_type)
        {
            try
            {
                //Get the pointer to the respective group
                m_redundancy_group = get_group();
                
                std::string componentName =this->get_element_name();
                std::string componentHost = (this->m_process_impl)->get_process_host_name();
                std::string fullname = componentName + "/" + componentHost;
                // Add the component to the group
                m_redundancy_group->add_component(fullname, this, ccmcomponent_def);
                
            }catch(CdmwPlatformMngt::GroupNotFound& )
            {
                CDMW_ERROR(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
              "GroupNotFound");
            }
        }
        
        try 
        {
            // Factory for component status change
            std::auto_ptr < ComponentStatusChangeFactory >
                compStatusChangeFactory (new ComponentStatusChangeFactory ());
            m_compStatusChangeFactory = compStatusChangeFactory.get ();

        std::auto_ptr <ComponentStateMachine> stateMachine ( new ComponentStateMachine (
                                                             this, "CcmComponet_impl"));
            
         m_stateMachine = stateMachine.get();
         
         // Add the states to the state machine
         m_stateMachine->addState ("Initial", M_initial);
         m_stateMachine->addState ("Initialising", M_initialising);
         m_stateMachine->addState ("Configuring", M_configuring);
         m_stateMachine->addState ("Initialised", M_initialised);
         m_stateMachine->addState ("Configured", M_configured);
         m_stateMachine->addState ("Stopping", M_stopping);
         m_stateMachine->addState ("Stopped", M_stopped);
         m_stateMachine->addState ("FailedError", M_failedError);
         m_stateMachine->addState ("FailedTimeout", M_failedTimeout);
         m_stateMachine->addState ("FailedInvalid", M_failedInvalid);
     
         stateMachine.release();
                   
         // Set the initial state
         m_stateMachine->setState ("Initial");


         //instance entity container ...

         std::auto_ptr <PlatformMngt::EntityContainer> actualEntityContainer (
          new PlatformMngt::EntityContainer(poa, m_element_repository.in(),
           m_element_path.in()));
                        
         m_rwEntityContainerServant = 
                 new RWEntityContainer_impl ( 
                    poa, actualEntityContainer.get(), m_element_repository.in());


         m_rwEntityContainer
                    = dynamic_cast <RWEntityContainer_impl*> (m_rwEntityContainerServant.in());

       // Add the entities of the process
        // if duplicated throw DuplicateEntity exception
        const CdmwPlatformMngt::EntityDefs& entities = ccmcomponent_def.entities;

        m_actualEntityContainer = actualEntityContainer.get();

        //unsigned int entityCount =  (ccmcomponent_def.entities).length();
        // Store entity initial values into default_entities
        //m_default_entitydefs =  new CdmwPlatformMngt::EntityDefs();
        //m_default_entitydefs->length(entityCount);


         actualEntityContainer.release();

        for (unsigned int entityIndex=0; entityIndex < entities.length(); entityIndex++)
        {
            // get entity definition (EntityDef is a structure, a reference on the
            // structure is returned by the [] operator)
            const CdmwPlatformMngt::EntityDef& entity_def = entities[entityIndex];

            // store values of the entityDef
            //(*m_default_entitydefs)[entityIndex] = entity_def;

            try
            {

                add_entity (entity_def);
            }
            catch(const CdmwPlatformMngt::EntityAlreadyExists&)
            {
                // in this case the servant will be deleted by application
                throw CdmwPlatformMngt::DuplicateEntity (entity_def.entity_name.in());
            }
        }


        //Add all Properties 
        unsigned int propCount = ccmcomponent_def.properties.length();
        for (unsigned int propIndex=0 ; propIndex < propCount ; propIndex++)
        {
            const CdmwPlatformMngt::PropertyDef& prop_def =
                (ccmcomponent_def.properties)[propIndex];
                        
            std::string propertyName(((ccmcomponent_def.properties)[propIndex]).property_name);
            CosPropertyService::PropertyModeType propertyMode =
                CosPropertyService::PropertyModeType(((ccmcomponent_def.properties)[propIndex]).mode_type);
                        
            try
            {       
                define_property_with_mode(propertyName.c_str() , prop_def.initial_value, propertyMode);
            }       
            catch (...) 
            {
                throw   CosPropertyService::MultipleExceptions(); 
            }
         
        }               

        //add pre-defined properties ...
        std::ostringstream property_name;
        CORBA::Any_var any_val = new CORBA::Any();
        try
        {
            *any_val <<= CdmwPlatformMngt::REDSTATE_NORG;
            property_name.str("");
            property_name << CdmwPlatformMngt::CDMW_PREFIX << CdmwPlatformMngt::REPLICATION_ROLE << std::ends;                  
            define_property_with_mode(property_name.str().c_str(), any_val.in(), CosPropertyService::fixed_normal);

            //CdmwPlatformMngt::RestartWarmth my_restart_warmth =
            //CdmwPlatformMngt::HOT_RESTART;
            property_name.str("");
            property_name << CdmwPlatformMngt::CDMW_PREFIX << CdmwPlatformMngt::DEFAULT_RESTART_WARMTH << std::ends;
            CORBA::Any_var any = new CORBA::Any();
              *any <<= CdmwPlatformMngt::HOT_RESTART;

        define_property_with_mode(property_name.str().c_str() , any.in(), CosPropertyService::fixed_normal);

            property_name.str("");
            property_name << CdmwPlatformMngt::CDMW_PREFIX << CdmwPlatformMngt::REQUESTED_RESTART_WARMTH<< std::ends;
            CORBA::Any_var any2;            
            get_attribute(CdmwPlatformMngt::DEFAULT_RESTART_WARMTH, any2.out(), CdmwPlatformMngt::PRE_DEFINED);
            define_property_with_mode(property_name.str().c_str() , any2.in(), CosPropertyService::fixed_normal);

        }
        catch (...)
        {
            throw CosPropertyService::MultipleExceptions ();
        }

        // Release the ownership of the temporary objects
        compStatusChangeFactory.release ();
        
        }
        catch (const std::bad_alloc&)
        {
             throw CORBA::NO_MEMORY(OrbSupport::NO_MEMORY, CORBA::COMPLETED_NO);
        }
         
         try
         {
             Cdmw::CommonSvcs::DataStore::DataStoreBase * ds =
                 Cdmw::CommonSvcs::DataStore::DataStoreRegistry::
                     Get_data_store (Configuration::Get_component_datastore_id ());
             m_component_ds =
                 dynamic_cast < Cdmw::PlatformMngt::ComponentDataStore * >(ds);

         }
         catch (const NotExistException & e)
         {
             CDMW_FATAL(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                                        "ComponentDatastore Not Found");
             CDMW_ASSERT (false);
         }
    
    }
    
    
    CcmComponent_impl::~CcmComponent_impl()
    {
        if (m_compStatusChangeFactory != 0)
        {
            m_compStatusChangeFactory->_remove_ref ();
        }
    }

    void CcmComponent_impl::set_indicator_relevant_status (CORBA::
                                                           Boolean
                                                           indicator_relevant_status)
      throw (CORBA::SystemException)
    {
      throw CORBA::NO_IMPLEMENT (OrbSupport::NO_IMPLEMENT, CORBA::COMPLETED_NO);
    }

    CORBA::ULong CcmComponent_impl::get_number_of_entities () throw (CORBA::
                                                                     SystemException)
    {
     return m_rwEntityContainer->get_number_of_entities();
    }

    void CcmComponent_impl::get_all_entity_names (CORBA::ULong how_many,
                                                  const CdmwPlatformMngt::
                                                  ElementNames_out
                                                  elem_names_out,
                                                  CdmwPlatformMngt::
                                                  ElementNamesIterator_out
                                                  elem_iter_out) throw (CORBA::
                                                                        SystemException)
    {
      m_rwEntityContainer->get_all_entity_names(how_many, elem_names_out, elem_iter_out );
     // throw CORBA::NO_IMPLEMENT (OrbSupport::NO_IMPLEMENT, CORBA::COMPLETED_NO);
    }

    CdmwPlatformMngt::Entity *
      CcmComponent_impl::get_entity (const char *name) throw ( CdmwPlatformMngt::EntityNotFound, 
                                                                CORBA::SystemException)
    {
      //throw CORBA::NO_IMPLEMENT (OrbSupport::NO_IMPLEMENT, CORBA::COMPLETED_NO);
       return m_rwEntityContainer->get_entity (name);
    }

    CORBA::Boolean CcmComponent_impl::get_entities (const CdmwPlatformMngt::
                                                    ElementNames & elem_names,
                                                    CdmwPlatformMngt::
                                                    Entities_out ent_out)
      throw (CORBA::SystemException)
    {
         return m_rwEntityContainer->get_entities(
            elem_names, ent_out );
    }

    void CcmComponent_impl::get_all_entities (CORBA::ULong how_many,
                                              CdmwPlatformMngt::
                                              Entities_out nentities,
                                              CdmwPlatformMngt::
                                              EntitiesIterator_out rest)
      throw (CORBA::SystemException)
    {
       m_rwEntityContainer->get_all_entities(
            how_many, nentities, rest ); 

    }
        
    CdmwPlatformMngt::Entity *
      CcmComponent_impl::add_entity (const CdmwPlatformMngt::
                                     EntityDef & ent_def) throw (CORBA::
                                                                 SystemException)
    {


    CdmwPlatformMngt::Entity_ptr ent_ptr =
        m_rwEntityContainer->add_entity(ent_def);

    //std:: stringstream str;
    //str<<"********* OK: ADDED ENTITY NAME="<<ent_def.entity_name;

    //std::string buffer(str.str());

    //CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG) , buffer)

    return ent_ptr ;
    }

    void CcmComponent_impl::get_attribute (const char *attr_name,
                                           CORBA::Any_out ret_attr,
                                           CdmwPlatformMngt::
                                           Owner flag) throw (CORBA::SystemException,
                                           CdmwPlatformMngt::AttributeNotFound)
    {

        std::string tmp1 (attr_name);
        if ( CdmwPlatformMngt::PRE_DEFINED == flag )
        {
            /*active the right methods */
            if (tmp1 == CdmwPlatformMngt::TECH_STATUS)
            {
                CdmwPlatformMngt::ComponentStatus stat;
                CORBA::String_var status_info;
                CORBA::Any_var tempany = new CORBA::Any;
    
                stat = get_status (status_info);
                tempany.inout () <<= stat;
                ret_attr = tempany._retn ();
            }
            else if (tmp1 == CdmwPlatformMngt::TECH_STATUS_INFO)
            {
                CdmwPlatformMngt::ComponentStatus stat;
                CORBA::String_var status_info;
                CORBA::Any_var tempany = new CORBA::Any;
         
                stat = get_status (status_info);
                tempany.inout () <<= status_info;
         
                ret_attr = tempany._retn ();
            }
            else {
                // Case of predefined property
                std::ostringstream property_name;
                property_name << CdmwPlatformMngt::CDMW_PREFIX << attr_name << std::ends;
                try
                {
                    CORBA::Any_var outany = 
                      get_property_value ( property_name.str().c_str() );
                    ret_attr = outany._retn();
                }           
                catch (CosPropertyService::PropertyNotFound)
                {
                    throw CdmwPlatformMngt::AttributeNotFound ();

               }
            }
        }
        else
        {
            /*case of user defined attribute */
            try
            {
                CORBA::Any * outany = get_property_value (attr_name);
                ret_attr = outany;
            }
            catch (CosPropertyService::PropertyNotFound)
            {
                throw CdmwPlatformMngt::AttributeNotFound ();
            }
        }
    }


    void CcmComponent_impl::set_attribute (const char *attr_name, 
                                           const CORBA::Any & attr,
                                           CdmwPlatformMngt::Owner flag) 
                                           throw (CORBA::SystemException,
                                                  CdmwPlatformMngt::AttributeNotFound,
                                                  CdmwPlatformMngt::ReadOnlyAttribute)

    {

        CORBA::Any_var myany = new CORBA::Any();
        *myany = attr; 
        
        if ( flag == CdmwPlatformMngt::PRE_DEFINED )
        {
            std::string tmp1 (attr_name);
            // check if the predefined property is readonly 
            if ((tmp1 == CdmwPlatformMngt::REQUESTED_RESTART_WARMTH) ||
                (tmp1 == CdmwPlatformMngt::REPLICATION_ROLE ))
            {
                throw CdmwPlatformMngt::ReadOnlyAttribute(); 
            }
        

        }

        try
        {
            set_attribute_internal ( attr_name, myany.in(), flag );
        }
        catch (Cdmw::BadOrderException& ex)
        {
        throw CORBA::BAD_PARAM(OrbSupport::BAD_PARAM, CORBA::COMPLETED_NO);
        }
        catch (...) {
            throw;
        }


    }


    CdmwPlatformMngt::ElementType CcmComponent_impl::
      get_element_type ()throw (CORBA::SystemException)
    {
      throw CORBA::NO_IMPLEMENT (OrbSupport::NO_IMPLEMENT, CORBA::COMPLETED_NO);
    }


    CdmwPlatformMngt::Children *
      CcmComponent_impl::get_children ()throw (CORBA::SystemException)
    {
        CdmwPlatformMngt::Children_var children = new CdmwPlatformMngt::Children;
        CORBA::ULong size_of_entities = get_number_of_entities();

    children->length(size_of_entities);

    CdmwPlatformMngt::Entities_var entities;
        CdmwPlatformMngt::EntitiesIterator_var entities_iterator;
        get_all_entities(size_of_entities, entities.out(), entities_iterator.out() );
            
        for (CORBA::ULong count = 0 ; count< size_of_entities; count++)
        {
           entities[count] -> _add_ref();
           children[count] = (entities[count])._retn();
        }
        return children._retn ();
    }


    CdmwPlatformMngt::ComponentSnapshot *
      CcmComponent_impl::get_snapshot ()throw (CORBA::SystemException)
    {
     CdmwPlatformMngt::ComponentSnapshot_var snapshot =
            new CdmwPlatformMngt::ComponentSnapshot;

            CdmwPlatformMngt::EntitiesSnapshot_var entitiesSnapshot
                    = new CdmwPlatformMngt::EntitiesSnapshot ();
         
      // Gets all the entities
      CdmwPlatformMngt::Entities_var entities;
      CdmwPlatformMngt::EntitiesIterator_var entitiesIt;
      CORBA::ULong nbr_of_entities;
         
      nbr_of_entities = get_number_of_entities ();
         
      get_all_entities (nbr_of_entities,
                        entities.out (), entitiesIt.out ());
         
      entitiesSnapshot->length (nbr_of_entities);
      for (CORBA::ULong i = 0; i < nbr_of_entities; i++)
                    {
                        entitiesSnapshot[i] = *(entities[i]->get_snapshot ());
                    }


              //Gets all the properties
                CosPropertyService::PropertiesIterator_var propertiesIt;
                CORBA::ULong nbr_of_properties;
                CdmwPlatformMngt::PropertiesSnapshot_var propertiesSnapshot =
                    new CdmwPlatformMngt::PropertiesSnapshot;
                nbr_of_properties = get_number_of_properties ();
                CosPropertyService::PropertyNames_var prop_names;
                CosPropertyService::PropertyNamesIterator_var prop_namesIt;
                get_all_property_names (nbr_of_properties, prop_names.out (),
                    prop_namesIt.out ());
                propertiesSnapshot->length (nbr_of_properties);
                for (CORBA::ULong i = 0; i < nbr_of_properties; i++)
                {
                    CdmwPlatformMngt::PropertySnapshot * propertySnapshot =
                        new CdmwPlatformMngt::PropertySnapshot ();
                    propertySnapshot->property_name =
                        CORBA::string_dup (prop_names[i]);

                    CORBA::Any prop_value;
                    prop_value = *(get_property_value (prop_names[i]));
                    CORBA::TypeCode_var tc;
                    tc = prop_value.type ();
                    switch (tc->kind ())
                    {
                    default:
                        break;
                    case CORBA::tk_string:
                        propertySnapshot->property_type =
                            CORBA::string_dup ("String");
                        break;
                    case CORBA::tk_short:
                        propertySnapshot->property_type = CORBA::string_dup ("Short");
                        break;
                    case CORBA::tk_float:
                        propertySnapshot->property_type = CORBA::string_dup ("Float");
                        break;
                    };
                    CosPropertyService::PropertyModeType mode =
                        get_property_mode (prop_names[i]);
                    propertySnapshot->property_value = prop_value;

                    if (mode == CosPropertyService::normal) 
                            propertySnapshot->property_mode = CORBA::string_dup ("Read/Write"); 
                    else 
                      if (mode == CosPropertyService::read_only)
                            propertySnapshot->property_mode = CORBA::string_dup ("Read/Only");
                      else 
                            propertySnapshot->property_mode = CORBA::string_dup ("Pre-defined");

                   propertySnapshot->property_value = prop_value;
                    propertiesSnapshot[i] = *(propertySnapshot);
                }

         
     // Create a component snapshot
     CORBA::String_var modeInfo;
     CORBA::String_var statusInfo;
     snapshot->element_path = element_path() ;
     snapshot->component_name = name() ;
     snapshot->component_deployment_plan = deployment_plan() ;
     snapshot->component_status = get_status (statusInfo.out ());
     snapshot->component_status_info = statusInfo;
     snapshot->component_entities = entitiesSnapshot.in() ;
     snapshot->component_properties = propertiesSnapshot.in ();
 
     return snapshot._retn();
    }


    CdmwPlatformMngt::ComponentStatus CcmComponent_impl::get_status (CORBA::String_out status_info)
              throw (CORBA::SystemException)
        {
         //FL   CDMW_SM_READER_LOCK_GUARD(*m_stateMachine);
            try
        { 
 
               CdmwPlatformMngt::ComponentRecord_var component_record = get_record_from_datastore();
             
               // get status
               CdmwPlatformMngt::ComponentStatus component_status = component_record->component_status;
             
               // get info associated to status
               status_info = CORBA::string_dup ((component_record->status_info).in());
                          
               return component_status;
         }
         catch (const std::bad_alloc&)
         {
             throw CORBA::NO_MEMORY (OrbSupport::NO_MEMORY, CORBA::COMPLETED_NO);
         }
    }


    char *CcmComponent_impl::instantiation_id () throw (CORBA::SystemException)
    {
      throw CORBA::NO_IMPLEMENT (OrbSupport::NO_IMPLEMENT, CORBA::COMPLETED_NO);
    }

/*
    Components::CCMObject *
      CcmComponent_impl::get_component ()throw (CORBA::SystemException)
    {
      throw CORBA::NO_IMPLEMENT (OrbSupport::NO_IMPLEMENT, CORBA::COMPLETED_NO);
    }
*/

    CdmwPlatformMngtBase::TimeStamp CcmComponent_impl::
      start_time ()throw (CORBA::SystemException)
    {
      throw CORBA::NO_IMPLEMENT (OrbSupport::NO_IMPLEMENT, CORBA::COMPLETED_NO);
    }


    CdmwPlatformMngtBase::TimeStamp CcmComponent_impl::
      stop_time ()throw (CORBA::SystemException)
    {
      throw CORBA::NO_IMPLEMENT (OrbSupport::NO_IMPLEMENT, CORBA::COMPLETED_NO);
    }


    char *CcmComponent_impl::get_component_server_name () throw (CORBA::
                                                                 SystemException)
    {
      throw CORBA::NO_IMPLEMENT (OrbSupport::NO_IMPLEMENT, CORBA::COMPLETED_NO);
    }


    CORBA::Boolean CcmComponent_impl::
      get_indicator_relevant_status ()throw (CORBA::SystemException)
    {
      throw CORBA::NO_IMPLEMENT (OrbSupport::NO_IMPLEMENT, CORBA::COMPLETED_NO);
    }

    Cdmw::PlatformMngt::RedundancyGroup_impl* CcmComponent_impl::
            get_group()
    throw (CORBA::SystemException,  CdmwPlatformMngt::GroupNotFound)
    {
        Cdmw::PlatformMngt::RedundancyGroup_impl* redundancy_group;
        Cdmw::PlatformMngt::Application_impl* appl = m_process_impl->get_application();  
    
        if (CdmwPlatformMngt::FT_GROUP_TYPE == m_redundancy_group_type)
        {
            redundancy_group = appl->getFTGroup(m_redundancy_group_name);
        }
        else if (CdmwPlatformMngt::LB_GROUP_TYPE == m_redundancy_group_type) 
        {
            redundancy_group = appl->getLBGroup(m_redundancy_group_name);
        }
        else
        {
            throw CdmwPlatformMngt::GroupNotFound();
        }
        return redundancy_group; 
    }
    
    CdmwPlatformMngt::RedundancyGroup *
      CcmComponent_impl::get_component_group ()throw (CORBA::SystemException)
    {
      if (0 != m_redundancy_group)
          return m_redundancy_group->_this();
      else
          return CdmwPlatformMngt::RedundancyGroup::_nil();
    }


    void CcmComponent_impl::remove_entity (const char *name) throw ( CdmwPlatformMngt::EntityNotFound, CORBA::
                                                                    SystemException)
    {
      throw CORBA::NO_IMPLEMENT (OrbSupport::NO_IMPLEMENT, CORBA::COMPLETED_NO);
    }

    char* CcmComponent_impl::deployment_plan () throw (CORBA::SystemException)
    {
        return CORBA::string_dup (m_deployment_plan);
    }

    void CcmComponent_impl::createStates()
        throw( OutOfMemoryException )
    {
        try
        {
            std::auto_ptr< ComponentInitial > initial(
                new ComponentInitial() );
            std::auto_ptr< ComponentInitialising > initialising(
                new ComponentInitialising() );
            std::auto_ptr< ComponentConfiguring > configuring(
                new ComponentConfiguring() );
            std::auto_ptr< ComponentInitialised > initialised(
                new ComponentInitialised() );
            std::auto_ptr< ComponentConfigured > configured(
                new ComponentConfigured() );
            std::auto_ptr< ComponentStopping > stopping(
                new ComponentStopping() );
            std::auto_ptr< ComponentStopped > stopped(
                new ComponentStopped() );
            std::auto_ptr< ComponentFailedError > failedError(
                new ComponentFailedError() );
/*            std::auto_ptr< ComponentFailedDeath > failedDeath(
                new ComponentFailedDeath() );
            std::auto_ptr< ComponentFailedDiscrepancy > failedDiscrepancy(
                new ComponentFailedDiscrepancy() );*/
            std::auto_ptr< ComponentFailedTimeout > failedTimeout(
                new ComponentFailedTimeout() );
            std::auto_ptr< ComponentFailedInvalid > failedInvalid(
                new ComponentFailedInvalid() );
            
            if( M_initial == NULL )
                M_initial = initial.release();
            if( M_initialising == NULL )
                M_initialising = initialising.release();
            if( M_configuring == NULL )
                M_configuring = configuring.release();
            if( M_initialised == NULL )
                M_initialised = initialised.release();
            if( M_configured == NULL )
                M_configured = configured.release();
            if( M_stopping == NULL )
                M_stopping = stopping.release();
            if( M_stopped == NULL )
                M_stopped = stopped.release();
            if( M_failedError == NULL )
                M_failedError = failedError.release();
/*            if( M_failedDeath == NULL )
                M_failedDeath = failedDeath.release();
            if( M_failedDiscrepancy == NULL )
                M_failedDiscrepancy = failedDiscrepancy.release();*/
            if( M_failedTimeout == NULL )
                M_failedTimeout = failedTimeout.release();
            if( M_failedInvalid == NULL )
                M_failedInvalid = failedInvalid.release();
        }
        catch( const std::bad_alloc& )
        {
            CDMW_THROW( OutOfMemoryException );
        }
    }
    
    void CcmComponent_impl::destroyStates()
    {
        if( M_initial != NULL )
        {
            delete M_initial;
            M_initial = NULL;
        }
        
        if( M_initialising != NULL )
        {
            delete M_initialising;
            M_initialising = NULL;
        }
        
        if( M_configuring != NULL )
        {
            delete M_configuring;
            M_configuring = NULL;
        }
        
        if( M_initialised != NULL )
        {
            delete M_initialised;
            M_initialised = NULL;
        }
        
        if( M_configured != NULL )
        {
            delete M_configured;
            M_configured = NULL;
        }
        
        if( M_stopping != NULL )
        {
            delete M_stopping;
            M_stopping = NULL;
        }
        
        if( M_stopped != NULL )
        {
            delete M_stopped;
            M_stopped = NULL;
        }
        
        if( M_failedError != NULL )
        {
            delete M_failedError;
            M_failedError = NULL;
        }
        
        if( M_failedTimeout != NULL )
        {
            delete M_failedTimeout;
            M_failedTimeout = NULL;
        }
        
        if( M_failedInvalid != NULL )
        {
            delete M_failedInvalid;
            M_failedInvalid = NULL;
        }
    }

    
    void CcmComponent_impl::setState( const char* stateName )
    {
    m_stateMachine->setState (stateName);
         
        if (m_redundancy_group != NULL)
    {
            //Updating the component status in to the group
            std::string componentName =this->get_element_name();
            std::string componentHost = (this->m_process_impl)->get_process_host_name();
            std::string fullname = componentName + "/" + componentHost;

            try
            {
                m_redundancy_group->update_component_status(fullname,stateName);

            }catch(CdmwPlatformMngt::ComponentNotFound& )
            {
                CDMW_ERROR(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                "ComponentNotFound");
            }
    }

    }
    
    CdmwPlatformMngt::ComponentStatus CcmComponent_impl::getStatus()
        throw( CORBA::SystemException )    
    {
        return m_stateMachine->getState()->status();
    }
    
    void CcmComponent_impl::initialise ( const CdmwPlatformMngtBase::StartupKind& startup_kind , const std::string& requested_deployment_plan)
        throw (CdmwPlatformMngt::IncompatibleStatus)
    {
        CDMW_SM_WRITER_LOCK_GUARD (*m_stateMachine);
        try
        {
            if (!strcmp(m_deployment_plan.in() , requested_deployment_plan.c_str()))
                m_stateMachine->getState ()->initialise (m_stateMachine, startup_kind);
        }
    
         catch( const CdmwPlatformMngt::IncompatibleStatus& e )
         {
         std::string message =  "Incompatible Status Exception for ";
         message+= name(); 
         CDMW_ERROR(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG), message.c_str());
         }

        catch (...)
        {
            // even in case of exception update record in datastore
            set_record_in_datastore ();
            CDMW_ERROR(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
 "CcmComponent_impl::initialise, Exception!! ");

            // rethrow exception
        //    throw;
        }
        /* TODO
        if ( CdmwPlatformMngt::COLD_RESTART==requested_restart_warmth() )
        {
            recover_default_properties ();
            recover_default_entities (m_rwEntityContainer, m_default_entitydefs);
        }
        */
        // update record in datastore
        set_record_in_datastore ();
    }
    
    void CcmComponent_impl::stopping() throw( CdmwPlatformMngt::IncompatibleStatus )
    {
    CDMW_SM_WRITER_LOCK_GUARD (*m_stateMachine);
    CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG), " stopping_components  on " << name());
        try
         {
             m_stateMachine->getState ()->set_stopping (m_stateMachine);
         }
        
          catch( const CdmwPlatformMngt::IncompatibleStatus& e )
          {
          CDMW_ERROR(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
        "Incompatible Status Exception");
        
          throw; 
          }
        
         catch (...)
         {
    
         CDMW_ERROR(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
        "CcmComponent_impl::stopping, Exception!! ");
        
         //rethrow exception
         throw;
         }
  }
    
    void CcmComponent_impl::stop() throw( CdmwPlatformMngt::IncompatibleStatus )
    {
    //MR
    //set_requested_restart_warmth ( get_default_restart_warmth () );
    CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG), " Stopping component " << name());
 
    CDMW_SM_WRITER_LOCK_GUARD (*m_stateMachine);
        try
          {
              m_stateMachine->getState ()->set_stop (m_stateMachine);
          }
     
           catch( const CdmwPlatformMngt::IncompatibleStatus& e )
           {
           CDMW_ERROR(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
         "Incompatible Status Exception");
         
           throw;
           }
         
          catch (...)
          {
      
              set_record_in_datastore();
     
              CDMW_ERROR(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
              "CcmComponent_impl::stop, Exception!! ");
         
              //rethrow exception
              throw;
          }
      set_record_in_datastore();
}


    void CcmComponent_impl::failed_error() throw( CdmwPlatformMngt::IncompatibleStatus )
    {
    CDMW_SM_WRITER_LOCK_GUARD (*m_stateMachine);
        try
          {
              m_stateMachine->getState ()->set_failed_error (m_stateMachine);
          }
     
           catch( const CdmwPlatformMngt::IncompatibleStatus& e )
           {
           CDMW_ERROR(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
         "Incompatible Status Exception");
         
           throw;
           }
         
          catch (...)
          {
          set_record_in_datastore();
     
          CDMW_ERROR(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
         "CcmComponent_impl::stop, Exception!! ");
         
          //rethrow exception
          throw;
          }
      set_record_in_datastore();
}
    
    
    void CcmComponent_impl::notifyStatusChangeEvent ()
    {
        try
        {     

            // Create the event
            CdmwPlatformMngtBase::EventHeader header
                = EventHeaderFactory::createHeader (CdmwPlatformMngtBase::INF);
    
            CdmwPlatformMngt::ProcessID_var processId = 
                new CdmwPlatformMngt::ProcessID;
            processId->application_name = 
                m_process_impl->get_application()->name();
            processId->process_name = m_process_impl->name();
            processId->host_name = m_process_impl->get_host()->name();
            
            CdmwPlatformMngt::ComponentStatusChange_var event
                = m_compStatusChangeFactory->create (header,
                m_process_impl->get_application()->get_system_name(),
                processId.in(),
                name(),
                m_stateMachine->getState ()->status (),
                m_statusInfo.in ());
                
            CORBA::String_var eventStr = event->to_string ();
            LogMngr::logMessage (eventStr.in ());
            
            // Perform the status change notification 
            // to supervision event notifier
            event->_add_ref(); 
            ((m_process_impl->get_application())->get_event_notifier())->addEvent(event);
            
            // Perform the status change notification to observer registered
            // with the Api in ManagedElement
            event->_add_ref(); 
            get_status_event_notifier()->add_event (event);
            
            event->_remove_ref();
            event._retn ();
        }
        catch (const std::bad_alloc &)
        {
            throw CORBA::NO_MEMORY (OrbSupport::NO_MEMORY, CORBA::COMPLETED_NO);
        }
        catch (...)
        {
            throw CORBA::INTERNAL (OrbSupport::INTERNAL, CORBA::COMPLETED_NO);
        }
   }
   
    
    void CcmComponent_impl::set_record_in_datastore ()
    {
        // test if the supervision is not fault tolerant or if it is, is it the primary?
        if (!Configuration::Is_fault_tolerant ()
        || Configuration::Is_primary ())
        {
            if (Configuration::Is_synchronising ())
            {
                std::
                    cout << "<<Component_impl::set_record_in_datastore() start >> "
                    << m_element_name.in () << std::endl;
            }
        
            CdmwPlatformMngt::ComponentRecord record;           
            record.redundancy_group_name = m_redundancy_group_name;
            record.redundancy_group_type = m_redundancy_group_type;
            record.deployment_plan = m_deployment_plan;
            record.name = m_name;
            record.component_status = (m_stateMachine->getState())->status();
        record.state_machine = CORBA::string_dup (m_stateMachine->getStateName ());
            // TODO add status info
        record.status_info = "" ;

            if (m_component_ds->contains (m_element_path.in ()))
            {
                m_component_ds->update (m_element_path.in (), record);
            }
            else
            {
                set_element_in_datastore ();
                m_component_ds->insert (m_element_path.in (), record);
            }

            if (Configuration::Is_synchronising ())
            {
                std::
                    cout << "<<Component_impl::set_record_in_datastore() done >> "
                    << m_element_name.in () << std::endl;
            }
        }
    }
      
    void CcmComponent_impl::synchro_with_datastore ()
    {

        std::cerr<<"CcmComponent_impl :synchro_with_datastore() "<<std::endl;
        Cdmw::PlatformMngt::ManagedElement_impl::synchro_with_datastore ();
        CdmwPlatformMngt::ComponentRecord_var component_record = get_record_from_datastore ();
        
        m_redundancy_group_name = component_record->redundancy_group_name;
        m_redundancy_group_type = component_record->redundancy_group_type;
        m_deployment_plan = component_record->deployment_plan;
        m_name = component_record->name;  
        m_stateMachine->setStateName (component_record->state_machine.in ());
        // TODO add status info
    //m_statusInfo = component_record->status_info;
    }


    void CcmComponent_impl::remove_record_in_datastore ()
    {
        // inhibition if backup
        if (!Configuration::Is_fault_tolerant ()
        || Configuration::Is_primary ())
        {
            try
            {
                m_component_ds->remove (m_element_path.in ());
            }
            catch (Common::NotFoundException &)
            {
            }
        }
    }


    CdmwPlatformMngt::ComponentRecord * CcmComponent_impl::get_record_from_datastore ()
    {
        CdmwPlatformMngt::ComponentRecord * component_record = NULL;

        try
        {
            component_record = m_component_ds->select (m_element_path.in ());
        }
        catch (Common::NotFoundException &)
        {   
            CDMW_FATAL(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                    "component record cannot be found in datastore : "
                    << m_element_path.in());
            CDMW_ASSERT (false);
        }

        return component_record;
    }

    
    

    bool CcmComponent_impl::doInitialisation() throw (CORBA::SystemException)
        {
    CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),"CcmComponent_impl:doInitialisation: " << name());
                                     
        bool ret = true;
        CdmwPlatformMngt::ProcessDelegateWrapper_var proc_delegate_w = m_process_impl ->getProcessDelegateWrapper();
            if (false == CORBA::is_nil(proc_delegate_w.in()) )
            {
                try 
                {
                CdmwPlatformMngt::ManagedComponentWrapper_var managed_comp_w = 
                        proc_delegate_w -> get_managed_component_wrapper ( name(), m_ccm_object.out());
                        if (false == CORBA::is_nil(managed_comp_w.in()) ) 
                        {
                        CdmwPlatformMngt::Timeout init_timeout = 0;           
                        managed_comp_w -> initialise_component_wrapper ( m_startup_kind, init_timeout );
                        }
                       else
                       {
                          CDMW_ERROR(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                                    "CcmComponent_impl: ManagedComponentWrapper is nil");
                           ret = false;
                       }
                        
                          
                }
                catch (const CORBA::SystemException &ex)
                {
                    CDMW_ERROR(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG), "CcmComponent_impl: doInitialisation exception!!");
                    throw;
                }
            }
            else 
            {
               CDMW_ERROR(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),
                         "CcmComponent_impl: ProcessDelegateWrapper is nil");
               ret = false;
            }
        return ret;
        }
  }                             // End namespace PlatformMngt
}                               // End namespace Cdmw
