/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX Sistemi Integrati. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2007. 
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include "platformapplicationlibrary/DeploymentManager.hpp"
#include "platformapplicationlibrary/DeploymentInfo.hpp"
#include "SystemMngt/platformlibrary/LogMngr.hpp"
#include "SystemMngt/idllib/CdmwPlatformMngtBase.stub.hpp"
#include <fstream>


#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/common/Options.hpp>
#include <Foundation/common/String.hpp>
#include <Foundation/common/Exception.hpp>
#include <Foundation/logging/LogManager.hpp>
#include <Foundation/ossupport/SystemProperties.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>


namespace Cdmw
{
  namespace PlatformMngt
  {

    using namespace Cdmw::OsSupport;
    using namespace Cdmw::OrbSupport;
    using namespace Cdmw::Common;
   
    static const char * CDMW_NODE_MANAGER_CORBANAME = "NameService#CdmwNodeManager"; 


    DeploymentManager* DeploymentManager::get_instance()
    {
        static DeploymentManager deployment_manager;
        return &deployment_manager;
    }
  
    void DeploymentManager::init(CORBA::ORB_ptr orb)
    { 
        if (!m_init)
        { 
            m_orb = CORBA::ORB::_duplicate (orb);
            m_init = true;
        }
    } 

    DeploymentManager::DeploymentManager(): m_init(false), m_node_manager_port_name("")
    {
        //Initialising the map
        m_map_deployment_executor.clear(); 
    }

    //TODO
    DeploymentManager::~DeploymentManager() throw()
    {
    }

    /*
    * Set the port and the name of node manager : port/name
    */ 
    void DeploymentManager::set_node_manager_port_name (const char * nodeManagerPortName)
    {
        m_node_manager_port_name = nodeManagerPortName;
    }
    
    /*
     * Recover Node Managers
     */
    void DeploymentManager::set_host_names(std::vector<std::string>* host_names)
    {
        int host_names_size = host_names->size();
        int counter = 0;
        // remove all DeploymentInfo with host name and deployment_plan
        while (counter < host_names_size)
        {
            std::string host_name = (*host_names)[counter];
            std::string corbaloc = "corbaname:iiop:";
            corbaloc+=host_name;
            corbaloc+=":";
            corbaloc+=m_node_manager_port_name;
            corbaloc+="/";
            corbaloc+=CDMW_NODE_MANAGER_CORBANAME;
            CORBA::Object_var obj =  m_orb->string_to_object(corbaloc.c_str());
                
            CdmwDnC::NodeManager_ptr node_manager_ptr = CdmwDnC::NodeManager::_narrow(obj.in());
            host_name = (*host_names)[counter];
            m_map_node_manager.insert(MapNodeManager::value_type(host_name, node_manager_ptr));
            ++counter;
        } 
        delete[] host_names;
    } 

    /*
     * Recover Node Manager
     */
    void DeploymentManager::set_node_manager_host_name(std::string host_name) throw (CORBA::SystemException)
    {
        CdmwDnC::NodeManager_ptr node_manager_ptr = CdmwDnC::NodeManager::_nil();
        try
        { 
            std::string corbaloc = "corbaname:iiop:";
            corbaloc+=host_name;
            corbaloc+=":";
            corbaloc+=m_node_manager_port_name;
            corbaloc+="/";
            corbaloc+=CDMW_NODE_MANAGER_CORBANAME;

            CORBA::Object_var obj =  m_orb->string_to_object(corbaloc.c_str());
            node_manager_ptr = CdmwDnC::NodeManager::_narrow(obj.in());

            if (!CORBA::is_nil(node_manager_ptr))
            {
                
                remove_node_manager_host_name(host_name);
                
                m_map_node_manager.insert(MapNodeManager::value_type(host_name, node_manager_ptr));
            }
        }
        catch(const CORBA::SystemException& ex)
        {
            CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG), ex._name() <<"  while trying to retrieve NodeManager on"<<host_name);
        }

        if (CORBA::is_nil(node_manager_ptr)) 
        {
            CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG), "NodeManager not found on "<<host_name);
            // the node manager has not been started or cant'be contacted 
            throw CORBA::BAD_INV_ORDER();
        }
    }

    /*
     * Remove Node Manager
     */
    void DeploymentManager::remove_node_manager_host_name(std::string host_name)
    {

        try
        {
            CdmwDnC::NodeManager_ptr node_manager_ptr = CdmwDnC::NodeManager::_nil();
            MapNodeManagerIterator m_map_node_manager_iterator_end = m_map_node_manager.end();
            MapNodeManagerIterator m_map_node_manager_iterator_found = m_map_node_manager.find(host_name);

            if( m_map_node_manager_iterator_found != m_map_node_manager_iterator_end)
            {
                node_manager_ptr = m_map_node_manager_iterator_found->second;
                CORBA::release(node_manager_ptr);
                m_map_node_manager.erase(m_map_node_manager_iterator_found);
            }
        }
        catch(const CORBA::SystemException& ex)
        {
            CDMW_FATAL(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG), ex._name() << " while releasing NodeManager on "<<host_name);
        }
    }

    CdmwDnC::NodeManager_ptr DeploymentManager::get_node_manager(const std::string& host_name)
    {
        CdmwDnC::NodeManager_ptr node_manager_ptr = CdmwDnC::NodeManager::_nil ();
        MapNodeManagerIterator m_map_node_manager_iterator_end = m_map_node_manager.end();
        MapNodeManagerIterator m_map_node_manager_iterator_found = m_map_node_manager.find(host_name);

        if( m_map_node_manager_iterator_found != m_map_node_manager_iterator_end)
        {
            node_manager_ptr = m_map_node_manager_iterator_found->second;
        }
        return node_manager_ptr;
    }  


    void DeploymentManager::set_node_deployment_info(const std::string& host_name, const std::string& deployment_plan, Cdmw::PlatformMngt::Application_impl *application) throw(CdmwDnC::StartError, CdmwDnC::PlanError, CORBA::SystemException) 
    {
         DeploymentInfo deployment_info;
         deployment_info.deployment_plan = deployment_plan;
         deployment_info.host_name = host_name;
 
         MapNodeDeploymentExecutor::iterator map_deployment_executor_it_found =  m_map_deployment_executor.find(deployment_info);
         MapNodeDeploymentExecutor::iterator map_deployment_executor_it_end = m_map_deployment_executor.end();
         if (map_deployment_executor_it_found == map_deployment_executor_it_end)
         {
             CdmwDnC::NodeManager_ptr node_manager_ptr = get_node_manager(host_name);
//             if(node_manager_ptr!=0)
             if (!CORBA::is_nil(node_manager_ptr))
             {
                 try
                 {
                      CdmwDnC::NodeApplicationManager_ptr node_application_manager_ptr = 
                          node_manager_ptr->prepare_plan(deployment_plan.c_str());
                      set_node_deployment_info(deployment_info, node_application_manager_ptr, application);  
                 }
                 catch(const CdmwDnC::StartError& start_error)
                 {
                     CDMW_FATAL(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG), "prepare_plan: an error occured at start-up " << deployment_plan.c_str());
                     throw;
                 }
                 catch(const CdmwDnC::PlanError& plan_error)
                 {
                     CDMW_FATAL(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG), "prepare_plan: an error occured reading DeploymentPlan " << deployment_plan.c_str());
                     throw;
                 }
                 catch(const CORBA::SystemException& ex)
                 {
                     CDMW_FATAL(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG), "prepare_plan: " << Cdmw::OrbSupport::OrbSupport::exception_to_string(ex).c_str());
                     throw; 
                 }
             catch(...)
                 {
                     CDMW_FATAL(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG), "prepare_plan: internal error" ) ;         
                     throw; 
                 }

            }
        }     
    }

    void DeploymentManager::set_node_deployment_info(const  DeploymentInfo& deployment_info, CdmwDnC::NodeApplicationManager_ptr node_application_manager_ptr,  Cdmw::PlatformMngt::Application_impl *application)
    {
         NodeDeploymentExecutor* node_deployment_executor = 
              new NodeDeploymentExecutor(deployment_info, node_application_manager_ptr, application);
          m_map_deployment_executor.insert(MapNodeDeploymentExecutor::value_type(deployment_info, node_deployment_executor));
    }   


    void DeploymentManager::set_node_deployment_infos(const std::string& host_name, 
                                                      const std::vector<std::string>* deployment_plans, 
                                                      Cdmw::PlatformMngt::Application_impl *application) throw(CdmwDnC::StartError, CdmwDnC::PlanError, CORBA::SystemException) 
    {

         CDMW_MUTEX_GUARD (m_mutex);

         int deployment_plans_size = deployment_plans->size();
         int counter = 0;

         while (counter < deployment_plans_size)
         {
             set_node_deployment_info(host_name, (*deployment_plans)[counter], application);
             ++counter;
         }
    }


    void DeploymentManager::remove_node_deployment_infos(const std::string& host_name, 
                                                         const std::vector<std::string>* deployment_plans)
    {
        CDMW_MUTEX_GUARD (m_mutex);

        DeploymentInfo deployment_info;
        deployment_info.host_name = host_name;   
        
        int deployment_plans_size = deployment_plans->size();
        int counter = 0;
        // remove all DeploymentInfo with host name and deployment_plan
        while (counter < deployment_plans_size)
        {
            std::string current_string = (*deployment_plans)[counter];
            deployment_info.deployment_plan = current_string;
            remove_node_deployment_info(deployment_info);
            ++counter;
        }
    }   
    
    void DeploymentManager::remove_node_deployment_info(const std::string& host_name, const std::string& deployment_plan)
    {
        DeploymentInfo deployment_info;
        deployment_info.deployment_plan = deployment_plan;
        deployment_info.host_name = host_name;
        remove_node_deployment_info(deployment_info);  
    }

    void DeploymentManager::remove_node_deployment_info(const  DeploymentInfo& deployment_info)
    {
        MapNodeDeploymentExecutorIterator map_deployment_executor_iterator_begin = m_map_deployment_executor.begin();
        MapNodeDeploymentExecutorIterator map_deployment_executor_iterator_end = m_map_deployment_executor.end();

        MapNodeDeploymentExecutorIterator map_deployment_executor_iterator_found = m_map_deployment_executor.find(deployment_info);
        // if found remove it
        if (map_deployment_executor_iterator_found != map_deployment_executor_iterator_end)
        {
            NodeDeploymentExecutor* node_deployment_executor = map_deployment_executor_iterator_found->second;
            
            // retrieve NodeManager
            MapNodeManagerIterator m_map_node_manager_iterator_begin = m_map_node_manager.begin();
            MapNodeManagerIterator m_map_node_manager_iterator_end = m_map_node_manager.end();
            MapNodeManagerIterator m_map_node_manager_iterator_found =  m_map_node_manager.find(deployment_info.host_name);
            CdmwDnC::NodeManager_ptr node_manager_ptr = m_map_node_manager_iterator_found->second;
            // destroy NodeApplicationManager

            try
            {
                CdmwDnC::NodeApplicationManager_var node_application_manager = node_deployment_executor->get_node_application_manager();
                delete node_deployment_executor;
                node_deployment_executor = 0;
                node_manager_ptr->destroy_manager(node_application_manager.in());

                CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG), "destroyed NodeApplicationManager with " << deployment_info.host_name << " and deployment plan= " << deployment_info.deployment_plan);
            }
            catch(...)
            {
                CDMW_FATAL(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG), "Exception while trying to destroy NodeApplicationManager");
            }
            m_map_deployment_executor.erase(map_deployment_executor_iterator_found);
            
        }
    }   
 
    void DeploymentManager::notify(const std::string& host_name, const std::string& deployment_plan) throw  (CdmwDnC::StartError)
    {
        
        CDMW_MUTEX_GUARD (m_mutex);

        DeploymentInfo deployment_info;
        deployment_info.host_name = host_name;
        deployment_info.deployment_plan = deployment_plan;

        // if all processes with the same host_name and deployment_plan 
        // call execute on NodeDeploymentExecutor
        update_node_deployment_executor(deployment_info);         
    }
    
    void DeploymentManager::notify(set<std::string> host_names_to_deploy, const std::string& deployment_plan) throw  (CdmwDnC::StartError)
    {
        
        CDMW_MUTEX_GUARD (m_mutex);

        set<std::string>::iterator host_names_to_deploy_it = host_names_to_deploy.begin();
        set<std::string>::iterator host_names_to_deploy_it_end   = host_names_to_deploy.end();
        while (host_names_to_deploy_it != host_names_to_deploy_it_end)
        {
            DeploymentInfo deployment_info;
            deployment_info.host_name = *host_names_to_deploy_it;
            deployment_info.deployment_plan = deployment_plan;
            ++host_names_to_deploy_it;
 
            // if all processes with the same host_name and deployment_plan
            // call execute on NodeDeploymentExecutor
            update_node_deployment_executor(deployment_info);
        }
     }

    void DeploymentManager::notify_process_component_event(const DeploymentInfo deployment_info, PROCESS_COMPONENT_EVENT event) throw (CdmwDnC::StartError)
    {
        CDMW_MUTEX_GUARD (m_mutex);

        MapNodeDeploymentExecutorIterator map_deployment_executor_iterator_begin = m_map_deployment_executor.begin();
        MapNodeDeploymentExecutorIterator map_deployment_executor_iterator_end = m_map_deployment_executor.end();

        MapNodeDeploymentExecutorIterator map_deployment_executor_iterator_found = m_map_deployment_executor.find(deployment_info);
        if (map_deployment_executor_iterator_found != map_deployment_executor_iterator_end)
        {

            NodeDeploymentExecutor* node_deployment_executor = map_deployment_executor_iterator_found->second;
            //TODO manage exception in node_deployment_executor
            try
            {
                if (PROCESS_COMPONENTS_COMPLETION_EVENT == event)
                    node_deployment_executor->decrease_counter();
                if (PROCESS_COMPONENTS_INITIALISATION_EVENT == event)
                    node_deployment_executor->increase_counter();
                if (PROCESS_COMPONENTS_SUSPENSION_EVENT == event)
                    node_deployment_executor->force_counter_to_zero();
            }
            catch(const CdmwDnC::StartError& start_error)
            {
                throw;
            }
        }
    }

    void DeploymentManager::notify_process_component_completion_event(const std::string deployment_plan, 
                                                                      const std::string & host_name) 
                                                                            throw (CdmwDnC::StartError)
    {  
         DeploymentInfo deployment_info;
         deployment_info.host_name = host_name;
         deployment_info.deployment_plan = deployment_plan;
         notify_process_component_event(deployment_info, PROCESS_COMPONENTS_COMPLETION_EVENT);
    }

    void DeploymentManager::notify_process_component_suspension_event(const std::string deployment_plan,
                                                            const std::string & host_name)
                                                                     throw (CdmwDnC::StartError)
    {
         DeploymentInfo deployment_info;
         deployment_info.host_name = host_name;
         deployment_info.deployment_plan = deployment_plan;
         notify_process_component_event(deployment_info, PROCESS_COMPONENTS_SUSPENSION_EVENT);
    }


    void DeploymentManager::notify_process_components_initialisation(const std::string deployment_plan, 
                                                                     const std::string & host_name) 
                                                                           throw (CdmwDnC::StartError)
    { 
         DeploymentInfo deployment_info;
         deployment_info.host_name = host_name;
         deployment_info.deployment_plan = deployment_plan;
         notify_process_component_event(deployment_info, PROCESS_COMPONENTS_INITIALISATION_EVENT);
    }
 

    void DeploymentManager::update_node_deployment_executor(const  DeploymentInfo deployment_info) throw (CdmwDnC::StartError)
    {
        MapNodeDeploymentExecutorIterator map_deployment_executor_iterator_begin = m_map_deployment_executor.begin();
        MapNodeDeploymentExecutorIterator map_deployment_executor_iterator_end = m_map_deployment_executor.end();

        MapNodeDeploymentExecutorIterator map_deployment_executor_iterator_found = m_map_deployment_executor.find(deployment_info);
        if (map_deployment_executor_iterator_found != map_deployment_executor_iterator_end)
        {
            NodeDeploymentExecutor* node_deployment_executor = map_deployment_executor_iterator_found->second;
            //TODO manage exception in node_deployment_executor
            try
            {
                CDMW_INFO(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG), "DeploymentManager scheduling deployment operation on host : "<< deployment_info.host_name <<  " and deployment plan: "<< deployment_info.deployment_plan);

                // start deployment operations  
                node_deployment_executor->start();
            }
            catch(const CdmwDnC::StartError& start_error)
            {
                throw;
            }
        }
    }

    void DeploymentManager::wait_all_node_deployment_executor_ended()
    {
         //CDMW_MUTEX_GUARD (m_mutex);
         MapNodeDeploymentExecutorIterator map_deployment_executor_iterator =   m_map_deployment_executor.begin();
         MapNodeDeploymentExecutorIterator map_deployment_executor_iterator_end =   m_map_deployment_executor.end();
         while(map_deployment_executor_iterator != map_deployment_executor_iterator_end)
         {
              NodeDeploymentExecutor* node_deployment_executor = map_deployment_executor_iterator->second;
              CDMW_DEBUG(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG), "joining thread deploying " << *(map_deployment_executor_iterator->first).deployment_plan.c_str()  << *(map_deployment_executor_iterator->first).host_name.c_str());

              if (THREAD_RUNNING == node_deployment_executor->get_status()) 
                  node_deployment_executor->join();

              CDMW_DEBUG(Cdmw::Logging::Logger::Get_Logger(CdmwPlatformMngtBase::PFMMNG),"finished joining thread deploying " << *(map_deployment_executor_iterator->first).deployment_plan.c_str()  << *(map_deployment_executor_iterator->first).host_name.c_str());

              map_deployment_executor_iterator++;
         }
    }

    void DeploymentManager::print()
    {
        MapNodeDeploymentExecutorIterator map_deployment_executor_iterator_begin = map_deployment_executor_iterator_begin = DeploymentManager::m_map_deployment_executor.begin();
    
        MapNodeDeploymentExecutorIterator map_deployment_executor_iterator_end = m_map_deployment_executor.end();

        MapNodeDeploymentExecutorIterator map_deployment_executor_iterator_found = map_deployment_executor_iterator_begin;
    
        while (map_deployment_executor_iterator_found != map_deployment_executor_iterator_end)
        {
            std::cerr <<  map_deployment_executor_iterator_found->first.deployment_plan<< " * "<<map_deployment_executor_iterator_found->second<< std::endl; 
            ++map_deployment_executor_iterator_found;
        }
    }
    
  }// closing namespaces
}


