/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX Sistemi Integrati. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2007. 
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include "Foundation/common/Exception.hpp"
#include "Foundation/osthreads/ThreadHandle.hpp"
#include "Foundation/osthreads/Mutex.hpp"
#include "Foundation/osthreads/MutexGuard.hpp"
#include "Foundation/orbsupport/CORBA.hpp"
#include "Foundation/ossupport/OS.hpp"
#include "Foundation/osthreads/Condition.hpp"
#include <Foundation/orbsupport/TypeTraits.hpp>

#include "SystemMngt/platformlibrary/MessageQueue.hpp"
#include "SystemMngt/platformlibrary/LogMngr.hpp"
#include "SystemMngt/platformlibrary/CommonLogMessageId.hpp"
#include "SystemMngt/platformlibrary/Configuration.hpp"
#include "SystemMngt/platformlibrary/DataStoreBaseDefinition.hpp"
#include "SystemMngt/idllib/CdmwPlatformMngtBase.stub.hpp"
#include "SystemMngt/platformlibrary/BoundSyncCall.hpp"
#include "SystemMngt/platformlibrary/EventNotifier.hpp"
#include "SystemMngt/idllib/CdmwPlatformMngtEvent.stub.hpp"
#include "SystemMngt/platformlibrary/PropTypes.hpp"
#include "SystemMngt/platformlibrary/PropUtils.hpp"
#include <tao/Messaging/Messaging.h>

#include <map>
#include <string>
#include <sstream>

#include "SystemMngt/platformlibrary/StatusEventNotifier.hpp"

namespace Cdmw
{
    namespace PlatformMngt
    {
       
        StatusEventNotifier::StatusEventNotifier (unsigned long callTimeout, CORBA::ORB_ptr the_orb)
        {
            m_bMustStop = false;
            m_bRunning = false;
            m_beventInQueue = false;
            m_callTimeout = callTimeout;
            m_evt_primary_key = 0;
            m_evt_seconds = 0;
            m_evt_microsec = 0;
            m_evt_index = 0;
            m_the_orb = CORBA::ORB::_duplicate (the_orb);
            m_event_notif_ds = 0;
            m_logger_pointer =
            Cdmw::Logging::Logger::Get_Logger (CdmwPlatformMngtBase::PFMMNG);
            CDMW_ASSERT (m_logger_pointer != 0);

            CDMW_DEBUG (m_logger_pointer, "Supervision Event Notifier Instance ");

            m_boundSyncCallFactory = BoundSyncCallFactory::getFactory ();


            // instance to datastore notifiers
            try
            {
            Cdmw::CommonSvcs::DataStore::DataStoreBase * ds =
                Cdmw::CommonSvcs::DataStore::DataStoreRegistry::
                Get_data_store (Configuration::
                        Get_event_notifier_ext_datastore_id ());
            m_event_notif_ds =
                dynamic_cast < Cdmw::PlatformMngt::EventNotifierDataStore * >(ds);
            }
            catch (const NotExistException & e)
            {
            CDMW_ASSERT (false);
            throw;
            }
            catch (std::bad_cast & e)
            {
            CDMW_FATAL (m_logger_pointer, e.what ());
            throw;
            }
        }

        StatusEventNotifier::~StatusEventNotifier ()throw ()
        {
            CORBA::release (m_the_orb);
            stop ();
            m_StatusEvQueue.destroy ();
        }


        std::string StatusEventNotifier::eventToString (CdmwPlatformMngt::Event*  event)
        {
            CORBA::String_var str = event->to_string ();

            std::string eventStr (str.in ());

            return eventStr;
        }


        void StatusEventNotifier::insertEventInDataStore (CdmwPlatformMngt::Event* event)
        {
            if (Configuration::Is_primary ())
            {
            // update the datastore
            // get the event key
            CdmwPlatformMngtBase::EventHeader & event_header =
                event->header ();

            try
            {
                m_event_notif_ds->insert (event_header.event_key, event);
            }
            catch (Common::AlreadyExistException &)
            {
                CDMW_WARN (Cdmw::Logging::Logger::
                       Get_Logger (CdmwPlatformMngtBase::PFMMNG),
                       "event notif record already exists in datastore : "
                       << event_header.event_key.
                       primary_key << ":" << event_header.event_key.
                       seconds << ":" << event_header.event_key.
                       microseconds << ":" << event_header.event_key.
                       counter_inx);

                // CDMW_ASSERT(false);
            }
            catch (...)
            {
                CDMW_WARN (Cdmw::Logging::Logger::
                       Get_Logger (CdmwPlatformMngtBase::PFMMNG),
                       "event notif record unexpected exception in insertion : "
                       << event_header.event_key.
                       primary_key << ":" << event_header.event_key.
                       seconds << ":" << event_header.event_key.
                       microseconds << ":" << event_header.event_key.
                       counter_inx);
            }
            }
        }

        void StatusEventNotifier::removeEventInDataStore (CdmwPlatformMngtBase::EventKey event_key)
        {
            if (Configuration::Is_primary ())
            {
            try
            {
                m_event_notif_ds->remove (event_key);
            }
            catch (Common::NotFoundException &)
            {
            }
            }
        }

        bool StatusEventNotifier::register_observer (std::string observer_name, std::string element_path,
                CdmwPlatformMngt::SupervisionObserver_ptr observer_ptr)
            throw (BadParameterException, OutOfMemoryException, CdmwPlatformMngt::ElementAlreadyExists)
        {
            //std::string logstr;
            //std::pair < MapObsForAll::iterator, bool > binsreturn;
            CdmwPlatformMngt::SupervisionObserver_ptr new_observer = CdmwPlatformMngt::SupervisionObserver::_nil ();

            if (m_bMustStop) {
                return false;
            }
            
            //bool bret = true;

            try
            {
                CDMW_MUTEX_GUARD (m_mutex);
    
                if (CORBA::is_nil (observer_ptr)) {
                    CDMW_THROW2 (BadParameterException, "observer_ptr", "NIL");
                }
                
                //check if the observer has already been registered
                //bool bool_ret = false;
                //std::string observer_name = obs_info.observer_name;
                //std::string target_element_path = element_path ;

                // return false if no key has the observer_name in its value
                bool bool_ret = 
                    PropUtils::check_obs_name_in_map_for_all(m_observers_regs, observer_name, element_path);

                // if the element is present throw an Element Already Present Exception 
                if (bool_ret) {
                    throw CdmwPlatformMngt::ElementAlreadyExists();
                }
                
                //new_observer = CdmwPlatformMngt::SupervisionObserver::_duplicate (observer_ptr);

                // if the element is not present insert it into observers map
                ObserverInfo obs_info;
                obs_info.observer_name = observer_name;
                obs_info.observer_ptr = CdmwPlatformMngt::SupervisionObserver::_duplicate(observer_ptr);
                //PropUtils::print_map_for_all(m_observers_regs);
        
                //binsreturn =
                m_observers_regs.insert (MapObsForAll::value_type (element_path, obs_info));
          
                //PropUtils::print_map_for_all(m_observers_regs);
            }
            catch (const std::bad_alloc &)
            {
                if (!CORBA::is_nil (new_observer)) {
                    CORBA::release (new_observer);
                }
                //bret = false;
                CDMW_THROW (OutOfMemoryException);
            }
            //return bret;
            return true;
        }


        bool StatusEventNotifier::unregister_observer (const std::string& observer_name, 
                                                            const std::string element_path)
                                                                throw (BadParameterException, 
                                                                       OutOfMemoryException,
                                                                       CdmwPlatformMngt::ObserverNotFound)
        {
            bool bret = false;
            bool bsync = false;
            MapObsForAll::iterator it_for_all;
    
            std::string target_element_path = element_path;
            CDMW_DEBUG (m_logger_pointer,
                "UnRegister the observer to Supervision Event Notifier");

            // make a copy of the observers in order to release the mutex
            m_mutex.lock ();
            MapObsForAll observers_regs (m_observers_regs);
            m_mutex.unlock ();
            // int to count the occurrencies in Multimap for Status or for All
            int observer_entries = 0;

            //StatusUtils::print_map_for_all(observers_regs);
 
            // find all observer occurrencies for the element_path
            observer_entries =    observers_regs.count (target_element_path);
            std::string observer_name_in_map = "";
            if (observer_entries!=0)
            { 
            it_for_all = observers_regs.find (target_element_path);
            observer_name_in_map = it_for_all->second.observer_name;
            }
            // if the observer is present in the map for all for that element path, erase the entry in the Map
            int found = 0;
            if((it_for_all != observers_regs.end ())&& (observer_name==observer_name_in_map))
            {    
                for (int j = 0; j < observer_entries; ++j)
                {
                if (it_for_all->second.observer_name == observer_name)
                {
                    //release the SupervisionObserver_ptr
                    CORBA::release(it_for_all->second.observer_ptr);
                    observers_regs.erase (it_for_all);
                    found++;
                }
                bsync = true;
                it_for_all++;
                }
            }
            if (found==0) throw CdmwPlatformMngt::ObserverNotFound(); 
            //StatusUtils::print_map_for_all(observers_regs);
            
            if (bsync)
            {
        // m_mutex.lock ();
            CDMW_MUTEX_GUARD (m_mutex);
            m_observers_regs = observers_regs;
            // m_mutex.unlock ();
            bret = true;
            }
            return bret;
        }
        
        std::vector<ObserverInfo> StatusEventNotifier::get_observers (std::string target_path)
        {
            std::vector<ObserverInfo> ret;
            std::pair<MapObsForAll::iterator, MapObsForAll::iterator> itp = m_observers_regs.equal_range(target_path);
            for (MapObsForAll::iterator it = itp.first; it != itp.second; ++it) {
                ret.push_back((*it).second);   
            }
            return ret;
        }
        
        bool StatusEventNotifier::is_registered_observer_empty ()
        {
            bool bret = false;
            m_mutex.lock ();
            bret = m_observers_regs.empty ();
            m_mutex.unlock ();
            return bret;
        }

        bool StatusEventNotifier::is_running ()
        {
            return m_bRunning;
        }

        bool StatusEventNotifier::add_event (CdmwPlatformMngt::Event *
                 event) throw (InternalErrorException)
        {   
            try
            {
            CDMW_MUTEX_GUARD (m_mutex);
            CDMW_DEBUG (m_logger_pointer, "add status event");

            if (m_bMustStop)
                return false;

            set_event_key (event);
            //insertEventInDataStore (event);
            m_StatusEvQueue.push (event);
            m_beventInQueue = true;
            return true;
            }
            catch (...)
            {
            CDMW_THROW (InternalErrorException);
            }
            return false;
        }

        void StatusEventNotifier::stop () throw ()
        {
            if (m_bMustStop)
            return;

            m_StatusEvQueue.destroy();
            this->join();
                                    
            CDMW_MUTEX_GUARD (m_mutex);
            CDMW_DEBUG (m_logger_pointer, "Stop Supervision Event Notifier ");
            m_bMustStop = true;
            m_beventInQueue = false;

        }
        

        void StatusEventNotifier::run () throw ()
        {
            std::string strace;
            std::string lpszfrm;

            //TypeNotificationCallResult result=SUCCESS;
            CdmwPlatformMngt::Event* StatusEvent = NULL;

            // SupervisionObserver_ptr to be notified
            CdmwPlatformMngt::SupervisionObserver_ptr observer_ptr = CdmwPlatformMngt::SupervisionObserver::_nil();
            if (!m_bRunning)
            m_bRunning = true;

            while (m_bRunning)
            { 
            bool bSync = false;
            std::string selement_path = "";
            std::string selement_path_chk = "";
            try
            {
                StatusEvent = 0;
                if (m_bMustStop)
                break;
                // Get the Event
                StatusEvent = m_StatusEvQueue.pop ();
                CDMW_DEBUG (m_logger_pointer,
                    "Supervision Event Notifier Event on Queue !");

                // make a copy of the observers in order to release the mutex
                m_mutex.lock ();
                /*ObserverElementMap */
                MapObsForAll observers_regs (m_observers_regs);
                m_mutex.unlock ();

                // get element_path affected
                selement_path = StatusEvent->element_path();
                // find map of registered observer for this element
                int observer_entries =
                observers_regs.count (selement_path);

                MapObsForAll::iterator it_for_all =
                observers_regs.find (selement_path);

                if (it_for_all != observers_regs.end ())
                {
                for (int j = 0; j < observer_entries; ++j)
                {
                    observer_ptr = it_for_all->second.observer_ptr;    
                    // Notify 
                    if (do_notification_call (observer_ptr, StatusEvent) != SUCCEED)
                    {
                    // The Observer has crashed, remove the entry
                    observers_regs.erase(it_for_all);
                    bSync=true;
                    }
                    it_for_all++;
    
                    if (CORBA::is_nil(observer_ptr))
                    {
                    observers_regs.erase(it_for_all);
                    bSync=true;
                    }
                }
                }
        
    
        
                ///////////////////////////////////////////////////////////////////
                // Sync registered observer map.
                ///////////////////////////////////////////////////////////////////
                if (bSync)
                {
                m_mutex.lock ();
                m_observers_regs = observers_regs;
                m_mutex.unlock ();
                bSync = false;
                }


                // remove event from datastore
                // get header
                CdmwPlatformMngtBase::EventHeader event_header =
                StatusEvent->header ();
                // request removing
                removeEventInDataStore (event_header.event_key);

                // deallocates the event
                StatusEventDeallocator::deallocate (StatusEvent);
            }
            catch (const MessageQueueDestroyedException&)
            {
                m_bMustStop = true;
            }
            catch (CORBA::SystemException &)
            {
                CDMW_DEBUG (m_logger_pointer, "System Exception...");
                StatusEventDeallocator::deallocate (StatusEvent);
                throw;
        }
            catch (...)
            {
                CDMW_DEBUG (m_logger_pointer, "System Exception...");
                StatusEventDeallocator::deallocate (StatusEvent);
            }
        }
    
            m_bRunning = false;

            CDMW_DEBUG (m_logger_pointer, "Supervision Event Notifier stopping...");
        }


        void StatusEventNotifier::set_event_key (CdmwPlatformMngt::Event * event)
        {

            // set the event key
            CdmwPlatformMngtBase::EventKey event_key;

            // set primary key
            event_key.primary_key = m_evt_primary_key;

            // get time val
            OsSupport::OS::Timeval time_val = OsSupport::OS::get_time ();

            event_key.seconds = static_cast < unsigned long >(time_val.seconds);
            event_key.microseconds =
            static_cast < unsigned long >(time_val.microseconds);

            bool evt_inx_incr = false;

            if (event_key.seconds < m_evt_seconds)
            {
            event_key.seconds = m_evt_seconds;
            event_key.microseconds = m_evt_microsec;
            evt_inx_incr = true;
            }

            else if (event_key.seconds == m_evt_seconds)
            {
            if (event_key.microseconds < m_evt_microsec)
            {
                event_key.microseconds = m_evt_microsec;
                evt_inx_incr = true;
            }
            else if (event_key.microseconds == m_evt_microsec)
            {
                evt_inx_incr = true;
            }
            }

            // if there is no difference between previous event time stamp
            // inrement the evt index to build an unique event key
            if (evt_inx_incr)
            {
            m_evt_index++;
            }
            else
            {
            // reset the index (time is different)
            // counter index in header is already set to 0
            m_evt_index = 0;
            }

            event_key.counter_inx = m_evt_index;

            // store event key

            // get header
            CdmwPlatformMngtBase::EventHeader & event_header = event->header ();

            // set event key in header
            event_header.event_key = event_key;

            // save the current time used for index
            m_evt_seconds = event_key.seconds;
            m_evt_microsec = event_key.microseconds;
        }



        CORBA::Object_ptr StatusEventNotifier::set_timeout_policy_on (CORBA::Object_ptr obj)
        {
            // Create the timeout policy
            CORBA::Any timeout;
            CORBA::Object_var capsule;

#if CDMW_ORB_VDR == orbacus
            CORBA::PolicyList policies (1);
            timeout <<= (CORBA::ULong) m_callTimeout;
            policies.length (1);
            policies[0] =
            m_the_orb->create_policy (OB::TIMEOUT_POLICY_ID, timeout);
            // Set the policy
            capsule =
            m_the_orb->_set_policy_overrides (policies, CORBA::ADD_OVERRIDE);

#elif CDMW_ORB_VDR == tao
            // Timeout success
            TimeBase::TimeT _timeout = ((unsigned long) (m_callTimeout * 1.0e4));
            CORBA::PolicyList policy_list;

            timeout <<= _timeout;
            // Set the policy
            policy_list.length (1);
            policy_list[0] =
            m_the_orb->
            create_policy (Messaging::RELATIVE_RT_TIMEOUT_POLICY_TYPE, timeout);

            CDMW_DEBUG (m_logger_pointer, "call obj->_set_policy_overrides");
            capsule =
            obj->_set_policy_overrides (policy_list, CORBA::SET_OVERRIDE);
#endif

            return capsule._retn ();
        }

        BoundSyncCallReturn   StatusEventNotifier::do_notification_call (
                                                                 CdmwPlatformMngt::SupervisionObserver_ptr observer_ptr,
                                                                 CdmwPlatformMngt::Event* StatusEvent) throw ()
        {
            CDMW_DEBUG (m_logger_pointer, "DoNotificationCall");
            BoundSyncCall *notifCall =
            m_boundSyncCallFactory->createEventNotificationCall (observer_ptr,
                                         StatusEvent,
                                         m_callTimeout);
            // performs the notification via the
            // bound synchronous call
            BoundSyncCallReturn ret = notifCall->execute ();

            delete notifCall;
            return ret;
        }



        BoundSyncCall* StatusEventNotifier::createEventNotificationCall (
                                                               CdmwPlatformMngt::SupervisionObserver_ptr   observer,
                                                               CdmwPlatformMngt::Event*   event)
                                                                    throw (OutOfMemoryException)
        {
            BoundSyncCall *notifCall =
            m_boundSyncCallFactory->createEventNotificationCall (observer,
                                         event,
                                         m_callTimeout);
            return notifCall;
        }
    }                // End namespace PlatformMngt
}                // End namespace Cdmw

