/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX Sistemi Integrati. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2007. 
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/ossupport/OS.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/orbsupport/StrategyList.hpp>
#include <Foundation/common/Options.hpp>
#include <Foundation/common/Exception.hpp>
#include "idllib/CdmwPlatformMngtSupervisionObserver.skel.hpp"
#include "idllib/CdmwPlatformMngtSystem.stub.hpp"
#include "idllib/CdmwPlatformMngtApplication.stub.hpp"
#include <SystemMngt/idllib/CdmwPlatformMngtSupervisionObserver.stub.hpp>
#include <FaultTolerance/ftreplicationmanager/InitUtils.hpp>
#include <FaultTolerance/idllib/FT.stub.hpp>
#include <SystemMngt/platformftproxy/InitUtils.hpp>
#include <SystemMngt/platformftproxy/SystemProxy.hpp>

#include <SystemMngt/platformvaluetypes/PropertyChange_impl.hpp>
#include <SystemMngt/idllib/CdmwPlatformMngtSupervisionObserver.stub.hpp>
#include "Foundation/common/System.hpp"
#include <platformftaccessorlibrary/SystemAccessor_impl.hpp>

#include "testFTpropertychange/SupervisionObserver_impl.hpp"

#include <SystemMngt/platformftproxy/AccessorLookupService.hpp>
#include <SystemMngt/platformftproxy/DefaultAccessorLookupService.hpp>

using namespace Cdmw;
using namespace Common;
using namespace PlatformMngt;
using namespace CdmwPlatformMngt;
using namespace std;

int main( int argc, char* argv[] )
{
    std::cerr << " Main Started " << std::endl;

    CORBA::ValueFactoryBase_var factory;
    CdmwPlatformMngt::Application_ptr application_test  = 0;
    CdmwPlatformMngt::Process_ptr     process_test      = 0;

    try {

    Cdmw::OrbSupport::StrategyList orb_strategies;
    orb_strategies.add_OrbThreaded();
    orb_strategies.add_PoaThreadPool( 10 );

    CORBA::ORB_var orb;
    orb = Cdmw::OrbSupport::OrbSupport::ORB_init(argc, argv,orb_strategies);

    CORBA::Object_var poa = orb->resolve_initial_references("RootPOA");
    PortableServer::POA_var rootPOA = PortableServer::POA::_narrow(poa.in());

    // activate the POA manager
    PortableServer::POAManager_var manager =  PortableServer::POAManager::_duplicate(rootPOA->the_POAManager());
    manager->activate();

    CdmwPlatformMngt::System_var system_test; 
    /*Create an Instance to Client Observer */
            
    std::string fault_manager_corbaloc = "corbaloc::localhost:4555/fault_manager";

    CORBA::Object_var fault_manager_obj = orb->string_to_object(fault_manager_corbaloc.c_str());
    if (CORBA::is_nil(fault_manager_obj.in()))
    {
            CDMW_THROW2(Cdmw::Exception, Cdmw::Exception::FATAL, "Invalid fault manager corbaloc");
    }

    CdmwFT::FTManager_var fault_manager;
    try
    {
            fault_manager = CdmwFT::FTManager::_duplicate(::CdmwFT::FTManager::_narrow(fault_manager_obj.in()));
    }
    catch (const CORBA::SystemException & ex)
    {
            std::cerr << "Exception while contacting FTManager: " << ex << "\n" << fault_manager_corbaloc << " is probably invalid" << std::endl;
    }
            
    CdmwFT::ReplicationManager_var replication_manager = fault_manager->get_replication_manager();

    // get from the replication manager the system accessor group
    // reference
    // from its group name
    ::FT::Name group_name;
    group_name.length(1);
    group_name[0].id = Cdmw::PlatformMngt::Configuration::Get_SystemAccessor_group_name().c_str();

    CORBA::Object_var system_accessor_group_obj =
            replication_manager->get_object_group_ref_from_name(group_name);

    CdmwFTSystMngt::SystemAccessor_var system_accessor =
            CdmwFTSystMngt::SystemAccessor::_narrow(system_accessor_group_obj.in());

    Cdmw::PlatformMngt::Proxy::SystemProxy *theSystemProxyHandle = NULL;
    const char * proxyName = "SystemMngtProxy";


    /* create lookup service */
    auto_ptr<Cdmw::PlatformMngt::Proxy::AccessorLookupService> 
        lookupService (new Cdmw::PlatformMngt::Proxy::DefaultAccessorLookupService(system_accessor.in()));

    theSystemProxyHandle = 
        Cdmw::PlatformMngt::Proxy::InitUtils::create_platformmngt_proxy (orb.in(), rootPOA.in(), proxyName, lookupService.get()); 
                    
    /* proxy takes over responsibility for lookup service */
            
    lookupService.release();

    system_test = theSystemProxyHandle->getReference();


    if (CORBA::is_nil (system_test.in ()))
    {
            // TEST_INFO ("Error in narrowing CdmwPlatformMngt::System");
            exit (-1);
    }

        CdmwPlatformMngt::Children_var children = new CdmwPlatformMngt::Children;
        children = system_test->get_children();
        CORBA::ULong size_of_children = children->length();

    // recoveer the Application APPL1
        application_test = system_test->get_application("APPL1");
        char * hostname  = getenv("HOSTNAME");
        process_test     = application_test->get_process("P11",hostname);

    // Prepare a Any value to set into define property
        CORBA::Any anyp_val;
        string sprop;
        int val=10;
        anyp_val <<= val;

        // begin a loop 
        while(1)
        {
          try
          {
            for (CORBA::ULong j = 0; j< size_of_children; j++)
            {
                children[j]->define_property("Property1",anyp_val);
                sleep(1);
                children[j]->define_property("Property2",anyp_val);
                sleep(1);
                children[j]->define_property("Property3",anyp_val);
                sleep(1);
            }
            process_test->define_property("Property1",anyp_val);
            sleep(1);
            system_test ->define_property("Property1",anyp_val);
            sleep(1);
            process_test->define_property("Property2",anyp_val);
            sleep(1);
            system_test->define_property("Property2",anyp_val);
            sleep(1);         
            process_test->define_property("Property3",anyp_val);
            sleep(1);         
            system_test->define_property("Property3",anyp_val);
          }
            catch(const CORBA::SystemException & e)
            {
                // if exception catched recover Children object 
                std::cerr << e._name()<< std::endl;
                children = new CdmwPlatformMngt::Children;
                children = system_test->get_children();
                size_of_children = children->length();
                process_test     = application_test->get_process("P11",hostname);
            }
        }
    }
    catch (CORBA::Exception& eCBex)
    {
    cout << "CORBA Exception :" << eCBex._name() << endl;
    return -1;
    }
    CORBA::release(application_test);
    CORBA::release(process_test);

    cout << "Exit process..." << endl;




}
