/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX Sistemi Integrati. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2007. 
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/* ===================================================================== */


#include <iostream>
#include <fstream>
#include <limits>

#include "test_global_supervision_properties/TestProp.hpp"

#include "Foundation/common/Options.hpp"
#include "Foundation/testutils/CORBATestManager.hpp"
#include "Foundation/orbsupport/OrbSupport.hpp"
#include "Foundation/ossupport/OS.hpp"

#include "Foundation/common/Exception.hpp"
#include "Foundation/common/System.hpp"
#include "Foundation/common/Assert.hpp"
#include "Foundation/orbsupport/StrategyList.hpp"
#include <SystemMngt/platformlibrary/Iterator.hpp>
#include "idllib/CdmwPlatformMngtSystem.stub.hpp"
#include "idllib/CdmwPlatformMngtPredefinedProperties.stub.hpp"


using namespace Cdmw;

using namespace Common;

using namespace PlatformMngt;

using namespace CdmwPlatformMngt;

CPPUNIT_TEST_SUITE_REGISTRATION (TestProp);

namespace Cdmw
{
  namespace PlatformMngt
  {
    
    TestProp::TestProp()
    {
    }

    void TestProp::setUp ()
    {
          try
          {
          //Getting system var
          std::string system_corbaloc = "corbaloc::localhost:21869/CdmwPlatformMngtSupervision";
          CORBA::ORB_var orb = CORBA::ORB::_duplicate (Cdmw::TestUtils::CORBATestManager::instance ()->get_ORB ());
          CORBA::Object_var system_obj = orb->string_to_object (system_corbaloc.c_str());
          m_system = CdmwPlatformMngt::System::_narrow (system_obj.in ());
          
          
          //Setting up the startupKind parameters
          state_retrieval = CdmwPlatformMngtBase::LAST_STATE;
          startup_mode = 0;
          specified_state_id = "";
          startupKind = new CdmwPlatformMngtBase::StartupKind();
          startupKind->state_retrieval = state_retrieval;
          startupKind->startup_mode = startup_mode;
          startupKind->specified_state_id  = CORBA::string_dup( specified_state_id.c_str() );
        
          //Setting the timeouts         
          long_timeout=7;
          short_timeout=3;
          
           
          }
          catch (const CORBA::SystemException& ex) 
          {
              std::cerr<< ex._name()<<std::endl;
              throw;
          }
          catch(...)
          {
          }
    }
    
    

    void TestProp::test_properties() 
    {
        std::string host2_name;
        std::string local_hostname = Cdmw::OsSupport::OS::get_hostname();
        
        //Getting the application APPL1 reference 
        CdmwPlatformMngt::Application_var application = m_system->get_application("APPL1");
        
        //References of system hosts
        CdmwPlatformMngt::Host_var Host1;
        CdmwPlatformMngt::Host_var Host2;       
        
        CORBA::ULong nb_hosts=m_system->get_number_of_hosts();
        CdmwPlatformMngt::Hosts_var hosts;
        CdmwPlatformMngt::HostsIterator_var rest;
        m_system->get_all_hosts(nb_hosts,hosts.out(),rest.out());
        
        //The current system has only two Hosts
        for (CORBA::ULong count = 0; count<nb_hosts; count++)
        {
            if( 0 != strcmp(hosts[count]->name(),local_hostname.c_str()) )
            {
                host2_name= hosts[count]->name();
                Host2= CdmwPlatformMngt::Host::_duplicate(hosts[count]);
            }else
            {
                Host1 = CdmwPlatformMngt::Host::_duplicate(hosts[count]);
            }
       }
        
        //Getting the processes references
        CdmwPlatformMngt::Process_var killer_process_host1 = 
            application->get_process("Killer_Supervision_host1",local_hostname.c_str());
        CdmwPlatformMngt::Process_var killer_process_host2 = 
            application->get_process("Killer_Supervision_host2",Host2->name());
        CdmwPlatformMngt::Process_var sup_start_host1 = 
            application->get_process("Start_Supervision_host1",local_hostname.c_str());
        CdmwPlatformMngt::Process_var sup_start_host2 = 
            application->get_process("Start_Supervision_host2",Host2->name());

        
        //The the primary supervision is on Host1
        check_properties (Host1,Host2);

        //Running the supervision Killer 
        
        TEST_INFO("Killing the supervision on "<<local_hostname);
        sleep(short_timeout);
        
        try
        {
            killer_process_host1->initialise(startupKind.in());                 
        }
        catch(const CORBA::SystemException& ex)
        {
            TEST_INFO("\nError running process..."<<__LINE__<<__FILE__);
        }
        
        sleep(short_timeout);
        
        //The the primary supervision is on Host2
        check_properties (Host2,Host1);
        
        //Restart the killed supervision on host1 
        TEST_INFO("Restarting the killed supervision on "<<local_hostname);
        sleep(short_timeout);

        try
        {
            sup_start_host1->initialise(startupKind.in());                  
        }
        catch(const CORBA::SystemException& ex)
        {
            TEST_INFO("\nError running Restart_Supervision process..."<<__LINE__<<__FILE__);
        }

        sleep(short_timeout);
        
        //The the primary supervision is on Host2
        check_properties (Host2,Host1);
        
        //Running the supervision Killer on Host2
        TEST_INFO("Killing the supervision on "<<Host2->name());
        sleep(short_timeout);
        try
        {
            killer_process_host2->initialise(startupKind.in());                 
        }
        catch(const CORBA::SystemException& ex)
        {
            TEST_INFO("\nError running process..."<<__LINE__<<__FILE__);
        }

        sleep(long_timeout);
        check_properties (Host1,Host2);
        
        //Restart the killed supervision on host2
        TEST_INFO("Restarting the killed supervision on "<<Host2->name());
        sleep(short_timeout);
        try
        {
            sup_start_host2->initialise(startupKind.in());                  
        }
        catch(const CORBA::SystemException& ex)
        {
            TEST_INFO("\nError running Restart_Supervision process..."<<__LINE__<<__FILE__);
        }
        sleep(short_timeout); 
               
 }
    
    
    
    
    
    void TestProp::check_properties (CdmwPlatformMngt::Host_var HostPrimary,CdmwPlatformMngt::Host_var HostSecondary)
    {
        CORBA::Any_var property_host_any = new CORBA::Any();
        const char* result = 0 ;
        CORBA::Any_var property_any = new CORBA::Any();

        
        //SYSTEM: Reading the current value of the system property GLOBAL_SUPERVISION_HOST 
        try 
        {

            m_system->get_attribute (CdmwPlatformMngt::GLOBAL_SUPERVISION_HOST, property_any.out(), CdmwPlatformMngt::PRE_DEFINED);
            property_any.in() >>= result;
            TEST_INFO( "Checking the current value of GLOBAL_SUPERVISION_HOST :" <<  result);
            CPPUNIT_ASSERT( ( 0 == strcmp(result,HostPrimary->name()) ) );
        }
        catch(CdmwPlatformMngt::AttributeNotFound)
        {
            TEST_INFO("\nERROR READING THE PROPERTY GLOBAL_SUPERVISION_HOST line:"<<__LINE__<<" file:"<<__FILE__);
            CPPUNIT_ASSERT(false);
        }

        //HOSTPRIMARY :Reading the current value of the property GLOBAL_SUPERVISION 
        TEST_INFO( "Checking the current value of the property GLOBAL_SUPERVISION on " << HostPrimary->name());
        CORBA::Boolean property_val;
        try {

            HostPrimary->get_attribute (CdmwPlatformMngt::GLOBAL_SUPERVISION, property_any.out(), CdmwPlatformMngt::PRE_DEFINED);
            property_any.in() >>=CORBA::Any::to_boolean(property_val);
            if(property_val) {
                TEST_INFO(HostPrimary->name()<< ":  GLOBAL_SUPERVISION is true " );
                CPPUNIT_ASSERT(true);
            } 
            else
            {
                TEST_INFO(HostPrimary->name()<< ":  GLOBAL_SUPERVISION is false " );
                CPPUNIT_ASSERT(false);}
        }
        catch(CdmwPlatformMngt::AttributeNotFound)
        {
            TEST_INFO(" ERROR READING THE PROPERTY GLOBAL_SUPERVISION line:"<<__LINE__<<" file:"<<__FILE__);
        }
        
        //HOSTSECONDARY :Reading the current value of the property GLOBAL_SUPERVISION 
        TEST_INFO( "Checking the current value of the property GLOBAL_SUPERVISION on " << HostSecondary->name());
        
        try {

            HostSecondary->get_attribute (CdmwPlatformMngt::GLOBAL_SUPERVISION, property_any.out(), CdmwPlatformMngt::PRE_DEFINED);
            property_any.in() >>=CORBA::Any::to_boolean(property_val);
                if(!property_val) {
                    TEST_INFO(HostSecondary->name()<< ":  GLOBAL_SUPERVISION is false " );
                    CPPUNIT_ASSERT(true);
                } 
                else
                {
                    TEST_INFO(HostSecondary->name()<< ":  GLOBAL_SUPERVISION is true " );
                    CPPUNIT_ASSERT(false);}
        }
        catch(CdmwPlatformMngt::AttributeNotFound)
        {
            TEST_INFO(" ERROR READING THE PROPERTY GLOBAL_SUPERVISION line:"<<__LINE__<<" file:"<<__FILE__);
        }
   }

}/* End namespace PlatformMngt */
}/* End namespace Cdmw */





