/* ========================================================================== *
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI.
 * Copyright (C) SELEX SISTEMI INTEGRATI 2004-2007. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * ========================================================================= */
package cdmw.clock;

import cdmw.ossupport.OS;

// import for ClockCatalog
import org.omg.CosClockService.ClockCatalogPackage.ClockEntriesHelper;
import org.omg.CosClockService.ClockCatalogPackage.ClockEntry;
import org.omg.CosClockService.ClockCatalogPackage.ClockEntryHolder;
import org.omg.CosClockService.ClockCatalogPackage.UnknownEntry;

import cdmw.orbsupport.ORBSupport;

import Cdmw.clock.ControlledClockExHelper;
import Cdmw.clock.ControlledClockEx;
import Cdmw.clock.ControlledExecutorHelper;
import Cdmw.clock.ControlledExecutor;



public class JNIClock
{

    public static class ClockThread extends Thread {
      
       private String[] args;
        /*
        *   ClockThread constructor
        */
        public ClockThread(String[] args) {
            String[] newStringArgs = new String[args.length +1 ];

            // it is needed for interaction between c++ and java 
            newStringArgs[0] = "dummyArgumentForJNI";
            

            System.arraycopy(args, 0, newStringArgs, 1, args.length);           
            
            
            this.args = newStringArgs ;
        }

        /*
        *   write IORs on file
        */
        private static void writeToFile(String[] iors,java.io.PrintWriter out) {
            for (int i=0; i< iors.length ; i++)
                    out.println(iors[i]);
        }
        /*
        *   run Clock thread
        */
        public void run() {

            //System.out.println(">>>>> [DEBUG] JNIClock:    InitOfCppClockService <<<<<<< ");
            initCppClock(args);
             while (true) 
             {
                try 
                {
                    OS.sleep(100000);
                   // System.out.println( ">>>>> [DEBUG] JNIClock: ^^^^^^^^^^Java ClockThread is alive  ^^^^^^^^^^ ");
                } catch(InterruptedException ie) {}

             }
        }
    }        
///////////////////////////////////////////////
    static final int LOCAL_CLOCK_ID                     = 0;
    static final int CLOCK_CATALOG_ID                   = 1;
    static final int MASTER_CLOCK_ID                    = 2;
    static final int POLICY_CONTROLLER_UPDATE_ADMIN_ID  = 3;
    static final int EXECUTOR_ID                        = 4;
    static final int FEDERATEDCONTROLLEDCLOCK_ID        = 5;
    static final int FEDERATEDCONTROLLEDEXECUTOR_ID     = 6;

    static ClockThread thr; 

    // load native library for initCppClock
    static {
        cdmw.ossupport.OS.loadLibrary("cdmwjniclockservice");
    }

    //obtain initialize C++ClockService
      public static native void initCppClock(String args[]);

    //obtain IORs corresponding to Clock Objects
      public static native String[] setIORsClockService();    

    //close C++ClockService
      public static native void closeCppClock();

    //close ClockService
    public static boolean close()
    {
        //System.out.println(">>>>> [DEBUG] JNIClock: ....close Clock  <<<<<<< ");
        closeCppClock();
        thr.stop();
        //System.out.println(">>>>> [DEBUG] JNIClock: ....threads stopped  <<<<<<< ");
        return true;
    }
    
    // register IORs on JAVA orb
    public static boolean init(org.omg.CORBA.ORB orb,String args[])
    {
        org.omg.CORBA.Object objClock = null;
       
        try {
        
            thr = new ClockThread (args);
            thr.start();
            
            //thr.join();
            //System.out.println(">>>>> [DEBUG] JNIClock: ....waiting for Clock init <<<<<<< ");
            OS.sleep(1);

            String[] iors = setIORsClockService();

            //System.out.println(">>>>> [DEBUG] JNIClock: ....create a Array of ClockObject(c++) <<<<<<< ");
            if (iors[LOCAL_CLOCK_ID].compareTo("NOT PRESENT") != 0)   
            {
                /////////////////////////////////////////////////////////////////
                // register LOCAL_CLOCK
                objClock = orb.string_to_object(iors[LOCAL_CLOCK_ID]);
                if (objClock == null) 
                
                {
                    System.out.println("LocalClockIOR not valid");
                    return false;
                }
                org.omg.CosClockService.Clock localclock =  
                    org.omg.CosClockService.ClockHelper.narrow(objClock);
                if (localclock == null) 
                {
                    System.out.println("Narrow failed : LocalClockIOR refers to different object: localclock is null");
                    return false;
                }
                ORBSupport.registerInitialReference((org.jacorb.orb.ORB)orb,"LocalClock", localclock);
                // bind  object to corbaloc
                ORBSupport.bindObjectToCorbaloc(orb,"LocalClock", localclock);
            }
        /////////////////////////////////////////////////////////////////
        // register CLOCK_CATALOG
        if (iors[CLOCK_CATALOG_ID].compareTo("NOT PRESENT") != 0)  
            {

                objClock = orb.string_to_object(iors[CLOCK_CATALOG_ID]);

                if (objClock == null) 
                {
                    System.out.println("ClockCatalogIOR not valid");
                    return false;
                }
                org.omg.CosClockService.ClockCatalog clockCatalog =  
                    org.omg.CosClockService.ClockCatalogHelper.narrow(objClock);
                if (clockCatalog == null) 
                {
                    System.out.println("Narrow failed : clockCatalogIOR refers to different object: clockCatalog is null");
                    return false;
                }
                // bind  object to corbaloc
             
                ORBSupport.registerInitialReference(orb,"ClockService", clockCatalog);
                ORBSupport.bindObjectToCorbaloc(orb, "ClockService", orb.string_to_object(iors[CLOCK_CATALOG_ID] ));
            }
         /////////////////////////////////////////////////////////////////
         // register MASTER_CLOCK
            {
            if (iors[MASTER_CLOCK_ID].compareTo("NOT PRESENT") != 0)  
                {
             objClock = orb.string_to_object(iors[MASTER_CLOCK_ID]);

                     if (objClock == null) 
                     {
                     System.out.println("MasterClockIOR not valid");
                     return false;
                     }
                     org.omg.CosClockService.Clock masterClock =  
                     org.omg.CosClockService.ClockHelper.narrow(objClock);
                     if (masterClock == null) 
                     {
                     System.out.println("Narrow failed : materClockIOR refers to different object: clockCatalog is null");
                     return false;
                     }
                     ORBSupport.registerInitialReference(orb,"MasterClock", masterClock);
        }  
            }
        /////////////////////////////////////////////////////////////////
        // register POLICY_CONTROLLER_UPDATE_ADMIN
         if (iors[POLICY_CONTROLLER_UPDATE_ADMIN_ID].compareTo("NOT PRESENT") != 0)  
            {
                objClock = orb.string_to_object(iors[POLICY_CONTROLLER_UPDATE_ADMIN_ID]);
                if (objClock == null) 
                {
                    System.out.println("PolicyControllerAdminIOR not valid");
                    return false;
                }
                Cdmw.clock.policy.ControllerUpdateAdmin policyControllerAdmin =  
                    Cdmw.clock.policy.ControllerUpdateAdminHelper.narrow(objClock);
                if (policyControllerAdmin == null) 
                {
                    System.out.println("Narrow failed : policyControllerAdminIOR refers to different object: clockCatalog is null");
                    return false;
                }   
                ORBSupport.registerInitialReference(orb,"PolicyControllerUpdateAdmin", policyControllerAdmin);
                // bind  object to corbaloc
                ORBSupport.bindObjectToCorbaloc(orb,"PolicyControllerUpdateAdmin", policyControllerAdmin);

            }
        /////////////////////////////////////////////////////////////////        
        // register EXECUTOR_ID
        if (iors[EXECUTOR_ID].compareTo("NOT PRESENT") != 0)   
            {
                objClock = orb.string_to_object(iors[EXECUTOR_ID]);
                if (objClock == null) 
                {
                    System.out.println("executor_ior not valid");
                    return false;
                }
                org.omg.CosClockService.PeriodicExecution.Executor executor = 
                    org.omg.CosClockService.PeriodicExecution.ExecutorHelper.narrow(objClock);

                if (executor == null) 
                {
                    System.out.println("narrow failed : executor refers to different object: executor is null");
                    return false;
                }
                ORBSupport.registerInitialReference(orb,"Jexecutor", executor);
            }
        /////////////////////////////////////////////////////////////////        
        /////////////////////////////////////////////////////////////////        
        // register FEDERATEDCONTROLLEDCLOCK_ID
        if (iors[FEDERATEDCONTROLLEDCLOCK_ID].compareTo("NOT PRESENT") != 0)   
            {
                objClock = orb.string_to_object(iors[FEDERATEDCONTROLLEDCLOCK_ID]);
                if (objClock == null) 
                {
                    System.out.println("federatedcontrolledclock_ior not valid");
                    return false;
                }
                ControlledClockEx  federatedcontrolledclock = 
                    ControlledClockExHelper.narrow(objClock);

                if (federatedcontrolledclock == null) 
                {
                    System.out.println("narrow failed : federatedcontrolledclock refers to different object: federatedcontrolledexecutor is null");
                    return false;
                }
                ORBSupport.registerInitialReference(orb,"FederatedControlledClock", federatedcontrolledclock);
            }
        /////////////////////////////////////////////////////////////////        
        /////////////////////////////////////////////////////////////////        
        // register FEDERATEDCONTROLLEDEXECUTOR_ID
        if (iors[FEDERATEDCONTROLLEDEXECUTOR_ID].compareTo("NOT PRESENT") != 0)  
            {
                objClock = orb.string_to_object(iors[FEDERATEDCONTROLLEDEXECUTOR_ID]);
                if (objClock == null) 
                {
                    System.out.println("federatedcontrolledexecutor_ior not valid");
                    return false;
                }
                ControlledExecutor federatedcontrolledexecutor = 
                    ControlledExecutorHelper.narrow(objClock);

                if (federatedcontrolledexecutor == null) 
                {
                    System.out.println("narrow failed : federatedcontrolledexecutor refers to different object: federatedcontrolledexecutor is null");
                    return false;
                }
                ORBSupport.registerInitialReference(orb,"FederatedControlledExecutor", federatedcontrolledexecutor);
            }
        /////////////////////////////////////////////////////////////////        


        }
        catch(InterruptedException ie) {}

        return true;
 }

}
