/* ========================================================================== *
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX Sistemi Integrati.
 * Copyright (C) SELEX Sistemi Integrati 2004-2006. All rights reserved.
 *
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 *
 *
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * ========================================================================= */

/**
 * @author Raffaele Mele <rmele@progesi.it>
 * @brief TimeAdmin - Federated Controlled Clock demo
 */

package cdmw.clock.federated;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;

import org.omg.CORBA.ORB;
import org.omg.CORBA.ORBPackage.InvalidName;
import org.omg.CosClockService.*;

import org.omg.CosClockService.ClockCatalogPackage.ClockEntry;
import org.omg.CosClockService.ClockCatalogPackage.UnknownEntry;
import org.omg.CosClockService.ControlledClockPackage.NotSupported;
import org.omg.CosClockService.ControlledClockHelper;

/**
 * @author rmele
 *
 */
public class TimeAdmin {
   // local variables
   private static ORB m_orb;

   private static ClockCatalog m_catalog;

   private static ControlledClock m_ctrlClock;

   // constant values
   static final int SUCCESS = 0;

   static final int ERROR = -1;

   static final int SET_TIME = 1;

   static final int SET_RATE = 2;

   static final int PAUSE = 3;

   static final int RESUME = 4;

   static final int EXIT = 5;

   public static long get_input(String echo) {
      if (echo != null) System.out.println(echo);
      try {
         BufferedReader inputUser
            = new BufferedReader(new InputStreamReader(System.in));
         String inputStr = inputUser.readLine();
         return Long.valueOf(inputStr).longValue();
      } catch (NumberFormatException ex)
      {
        System.out.println(":>> Wrong choice!!");
        return 0;
      } catch (java.io.IOException ex) {
         ex.printStackTrace();
         return 0;
      }
   }

   public static void main(String[] args) {

      m_orb = org.omg.CORBA.ORB.init(args, null);
      cdmw.clock.CdmwCosClock.init(m_orb, args);

      org.omg.CORBA.Object obj = null;
      try {
         obj = m_orb.resolve_initial_references("ClockService");
      } catch (InvalidName e1) {
         e1.printStackTrace();
         System.exit(ERROR);
      }
      if (obj == null) {
         System.out.println("Error : Unable to resolve ClockService");
      }
      m_catalog = ClockCatalogHelper.narrow(obj);
      if (m_catalog == null) {
         System.out.println("Error : Unable to resolve clock catalog");
      }

      ClockEntry entry;
      try {

         entry = m_catalog.get_entry("FederatedControlledClock");
         m_ctrlClock = ControlledClockHelper.narrow(entry.subject);
      } catch (UnknownEntry ex) {
         System.out.println("ClockCatalog::UnknownEntry");
      }

      if (m_ctrlClock == null) {
         System.err.println("Unable to resolve FederatedControlledClock: Aborting!");
         System.exit(ERROR);
      }
      //System.out.println(m_orb.object_to_string(m_ctrlClock));
      BufferedReader inputUser = new BufferedReader(new InputStreamReader(System.in));
      String inputStr;

      try {
         long choice = 0;
         Runtime p = Runtime.getRuntime();

         while (choice != EXIT) {
            switch ((int) choice) {
            case SET_TIME: {
               System.out.println("Set Time >> Enter New Time: ");
               inputStr = inputUser.readLine();
               long the_time = Long.valueOf(inputStr).longValue();
               m_ctrlClock.set(the_time);
               break;
            }
            case SET_RATE: {
               System.out.println("Set Rate >> Enter New Rate: ");
               inputStr = inputUser.readLine();
               float the_rate = Float.valueOf(inputStr).floatValue();
               m_ctrlClock.set_rate(the_rate);
               break;
            }
            case PAUSE: {
               System.out.println(">> Pause requested.");
               m_ctrlClock.pause();
               break;
            }
            case RESUME: {
               System.out.println(">> Resume requested.");
               m_ctrlClock.resume();
               break;
            }
            case EXIT:
            default:
               break;
            }

            try
            {
               p.exec("clear");
            } catch (IOException e) {
               e.printStackTrace();
            }
            choice = get_input("\t[1] Set Time\n" + "\t[2] Set Rate\n"
                  + "\t[3] Pause\n" + "\t[4] Resume\n" + "\t[5] Exit\n"
                  + "Enter your choice: ");
         }
      } catch (IOException ex) {
         ex.printStackTrace();
         System.exit(ERROR);
      } catch (NotSupported ex) {
         ex.printStackTrace();
         System.exit(ERROR);
      }
   }
}
