/* ========================================================================== *
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX SISTEMI INTEGRATI.
 * Copyright (C) SELEX SISTEMI INTEGRATI 2004-2007. All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * ========================================================================= */
package cdmw.lb.lbinit;

import java.util.List;
import org.omg.CORBA.*;
import org.omg.CORBA.Object;
import org.omg.CORBA.ORBPackage.InvalidName;
import org.omg.CosNaming.NamingContextPackage.CannotProceed;
import org.omg.CosNaming.NamingContextPackage.NotFound;
import org.omg.PortableInterceptor.*;
import org.jacorb.orb.ParsedIOR;
import org.jacorb.orb.CDRInputStream;
// import for LOGGING
import static cdmw.logging.Logger.*;
import static cdmw.logging.FunctionLogger.*;
import cdmw.ossupport.OS;


import com.thalesgroup.CdmwNamingAndRepository.NoNameDomain;

import cdmw.orbsupport.ExceptionMinorCodes;
//import cdmw.cdmwinit.InitUtils;

public class ServerRequestInterceptor extends org.omg.CORBA.LocalObject
        implements org.omg.PortableInterceptor.ServerRequestInterceptor
{
    /**
     * 
     */
    private static final long       serialVersionUID = 1L;

    org.omg.CORBA.ORB               m_orb;

    String                          m_name;
    
    CdmwLB.LBGroupManager M_gm = null;

    public ServerRequestInterceptor(org.omg.CORBA.ORB orb, String name)
    {
        m_orb = orb;
        m_name = name;
        
        CDMW_LOG_FUNCTION(LBInitLogger.GetLBInitLogger(), "");
        //Use LBGroup Manager
        String LB_GM_str = System.getProperty("cdmw.lb.LBGroupManager");
        //Check if --LBGroupManager option is present
        if(!LB_GM_str.equalsIgnoreCase("no") && !LB_GM_str.equalsIgnoreCase("yes"))
        {
            
            org.omg.CORBA.Object obj = m_orb.string_to_object(LB_GM_str);
            if (obj == null)
            {
                //System.out.println("Error: LBGroupManager - null reference");
                CDMW_DEBUG(LBInitLogger.GetLBInitLogger(), 
                               "    Error: LBGroupManager - null reference    ",  
                               new Throwable());
                
                
                throw new org.omg.CORBA.INTERNAL(
                       cdmw.orbsupport.ExceptionMinorCodes.INTERNALLoadBalancingError
                      , org.omg.CORBA.CompletionStatus.COMPLETED_NO);
            }
            M_gm = CdmwLB.LBGroupManagerHelper.narrow(obj);
        }
        else
        {
            org.omg.CORBA.Object objref =  obtainLBgmFromRepository();
            if (objref == null)
            {
                //System.out.println("Error: LBGroupManager - null reference");
                CDMW_DEBUG(LBInitLogger.GetLBInitLogger(), 
                               "    Error: LBGroupManager - null reference    ",  
                               new Throwable());
                
                
                throw new org.omg.CORBA.INTERNAL
                      (cdmw.orbsupport.ExceptionMinorCodes.INTERNALLoadBalancingError
                      , org.omg.CORBA.CompletionStatus.COMPLETED_NO);
            }            
            M_gm = CdmwLB.LBGroupManagerHelper.narrow(objref);
            //----------------
        }
    }
    

public org.omg.CORBA.Object  obtainLBgmFromRepository()
{
            
            //ACCESS TO NAMING
            // ======================================================== 
            //        Obtain LB Group Manager from CdmwNamingAndRepository                        
            // ========================================================
            String NamRapos_str = System.getProperty("cdmw.lb.namingRepos");
            // ===================================================
            // Get the naming and repository
            //     from orb
            // ===================================================
            com.thalesgroup.CdmwNamingAndRepository.Repository repository = null;
            org.omg.CORBA.Object objName = null;
            objName = m_orb.string_to_object(NamRapos_str);                          
            if (objName == null)
            {
                //System.out.println("Error: NamingRepository - null reference");
                CDMW_DEBUG(LBInitLogger.GetLBInitLogger(), 
                               "    Error: NamingRepository - null reference    ",  
                               new Throwable());
                
                throw new org.omg.CORBA.INTERNAL
                    (cdmw.orbsupport.ExceptionMinorCodes.INTERNALLoadBalancingError
                     , org.omg.CORBA.CompletionStatus.COMPLETED_NO);
            }
            repository = com.thalesgroup.CdmwNamingAndRepository.RepositoryHelper.narrow(objName);            

            // ======================================================   
            // Initialise the repository interface
            //     "CDMW" has been chosen as the default domain name
            // ======================================================
            try {
                cdmw.namingandrepository.RepositoryInterface.init ("CDMW", repository);
            } catch (NoNameDomain e1) {
                // TODO Auto-generated catch block
                //e1.printStackTrace();
                CDMW_INFO(LBInitLogger.GetLBInitLogger(), 
                           "WORNING : CDMW domain is not defined", new Throwable());
            } catch (com.thalesgroup.CdmwNamingAndRepository.InvalidName e1) {
                // TODO Auto-generated catch block
                // e1.printStackTrace();
                CDMW_INFO(LBInitLogger.GetLBInitLogger(), 
                "WORNING : CDMW not valid name", new Throwable());

            }
               
             // ====================================================== 
             // Get Naming Interface of LBGROUPMANAGER name domain 
             // ======================================================;
            cdmw.commonsvcs.naming.NamingInterface ni = null;
            try {
                ni = cdmw.namingandrepository.RepositoryInterface.getDomainNamingInterface("lb_group_manager/object_groups");
            } catch (NoNameDomain e1) {                                                     
                // TODO Auto-generated catch block
                //e1.printStackTrace();
                CDMW_INFO(LBInitLogger.GetLBInitLogger(), 
                           "WARNING : lb_group_manager/object_groups domain is not defined", 
                           new Throwable());
            } catch (com.thalesgroup.CdmwNamingAndRepository.InvalidName e1) {
                // TODO Auto-generated catch block
                // e1.printStackTrace();
                CDMW_INFO(LBInitLogger.GetLBInitLogger(), 
                           "WARNING : lb_group_manager/object_groups domain is not defined", 
                           new Throwable());                
            }
                
            // ===================================================
            // Get the GroupManager object from repository
            // ===================================================
            org.omg.CORBA.Object objref = null;
            try {
                objref = ni.resolve ("LBGroupManagerGroup");
            } catch (NotFound e1) {
                // TODO Auto-generated catch block
               // e1.printStackTrace();
                CDMW_INFO(LBInitLogger.GetLBInitLogger(), 
                           "WARNING : LBGroupManagerGroup not found", 
                           new Throwable()); 
            } catch (CannotProceed e1) {
                // TODO Auto-generated catch block
                //e1.printStackTrace();
                CDMW_INFO(LBInitLogger.GetLBInitLogger(), 
                           "WARNING : LBGroupManagerGroup not found", 
                           new Throwable());                 
            } catch (org.omg.CosNaming.NamingContextPackage.InvalidName e1) {
                // TODO Auto-generated catch block
                //e1.printStackTrace();
                CDMW_INFO(LBInitLogger.GetLBInitLogger(), 
                           "WARNING : LBGroupManagerGroup is an InvalidName", 
                           new Throwable());                  
            }
            return objref;
}

    public void destroy()
    {
    };

   
    public String name()
    {
        return "LBServerRequestInterceptor";
    }

    public void send_exception(ServerRequestInfo ri) throws ForwardRequest
    {
        //System.out.println("    send exception()  LBServer JAVA  ");
        CDMW_DEBUG(LBInitLogger.GetLBInitLogger(), " send exception()", new Throwable());

    }

    public void send_reply(ServerRequestInfo ri)
    {
        //System.out.println("    send reply()    ");
        CDMW_DEBUG(LBInitLogger.GetLBInitLogger(), " send reply()", new Throwable());
    }

    public void receive_request(ServerRequestInfo ri) throws ForwardRequest
    {
        //System.out.println("    receive_request()    ");
        CDMW_DEBUG(LBInitLogger.GetLBInitLogger(), "  receive_request() ", new Throwable());
    }

    public void receive_request_service_contexts(ServerRequestInfo ri)
            throws ForwardRequest
    {
        //System.out.println("    receive request service contexts()    ");
        CDMW_DEBUG(LBInitLogger.GetLBInitLogger(), 
                   "   receive request service contexts()    ", new Throwable());
        try
        {
            //
            // Get lb_GROUP_VERSION request service context
            //
            org.omg.IOP.ServiceContext sc = ri
                    .get_request_service_context(CdmwLB.LB_GROUP_VERSION.value);

            CdmwLB.LBGroupVersionServiceContext lb_service_ctx;

            if (sc.context_id == CdmwLB.LB_GROUP_VERSION.value)
            {

                byte[] data = sc.context_data;
                CDRInputStream in = new CDRInputStream(m_orb, data);
                // LSB representation
                in.setLittleEndian(in.read_boolean());
                lb_service_ctx = CdmwLB.LBGroupVersionServiceContextHelper
                        .read(in);
 
                CDMW_DEBUG(LBInitLogger.GetLBInitLogger(), 
                            "   lbGroupVersionServiceContext:  Object group ref version:  "
                            + lb_service_ctx.object_group_ref_version
                            + "  Object group Id: "
                            + lb_service_ctx.object_group_id,
                            new Throwable());    

               
                 
                if (M_gm.equals(null))
                {
                    CDMW_DEBUG(LBInitLogger.GetLBInitLogger(), 
                               " Error: LBGroupManager - null reference", 
                               new Throwable());
                               
                   
                    throw new org.omg.CORBA.INTERNAL(cdmw.orbsupport.ExceptionMinorCodes.INTERNALLoadBalancingError
                            , org.omg.CORBA.CompletionStatus.COMPLETED_NO);
                          
                          
                            
                }
                org.omg.CORBA.Object object_group = null;
                long new_version = 0;

                            
                try 
                {
                   object_group = M_gm
                            .get_object_group_ref_from_id(lb_service_ctx.object_group_id);
                }
                catch (org.omg.PortableGroup.ObjectGroupNotFound nf) {
                    CDMW_DEBUG(LBInitLogger.GetLBInitLogger(), 
                                   "Object Group Not Found", 
                                   new Throwable());                
                }
                catch (org.omg.CORBA.TRANSIENT t) 
                {
                    
                    CDMW_DEBUG(LBInitLogger.GetLBInitLogger(), 
                                   "The reference to LB Group Manager is updated", 
                                   new Throwable());
                    // In case of  LBGroupManagerFT
                    // If there is a LBGM fault, the LBGroupManager reference is not valid, so it has recovered
                    // into CDMW.Repository

                    org.omg.CORBA.Object objref =  obtainLBgmFromRepository();
                    M_gm = CdmwLB.LBGroupManagerHelper.narrow(objref);
                    try 
                    {                     
                        object_group = M_gm
                            .get_object_group_ref_from_id(lb_service_ctx.object_group_id);
                    }       
                    catch (org.omg.PortableGroup.ObjectGroupNotFound e)
                    {
                        e.printStackTrace();
                    }
                    catch (org.omg.CORBA.TRANSIENT secondt)
                    {

                       
                        CDMW_ERROR(LBInitLogger.GetLBInitLogger(), 
                                       "LB Group Manager is unreachable", 
                                       new Throwable());                   
                       /*throw new org.omg.CORBA.INTERNAL(
                            ExceptionMinorCodes.INTERNALLoadBalancingError,
                            org.omg.CORBA.CompletionStatus.COMPLETED_NO);*/
                        throw new org.omg.CORBA.TRANSIENT( ExceptionMinorCodes.TRANSIENT
                      , org.omg.CORBA.CompletionStatus.COMPLETED_NO);    
                    }
                } 
                                
                try
                {
 
                    new_version = M_gm
                            .get_object_group_version_from_ref(object_group);
                }
                catch (org.omg.PortableGroup.ObjectGroupNotFound e)
                {
                    e.printStackTrace();
                }
                catch (org.omg.CORBA.TRANSIENT t)
                {

                   //System.out.println("ERROR :  LB Group Manager reference is not valid! ");
                    CDMW_ERROR(LBInitLogger.GetLBInitLogger(), 
                                   "The reference to LB Group Manager ", 
                                   new Throwable());                   
                   throw new org.omg.CORBA.INTERNAL(
                        ExceptionMinorCodes.INTERNALLoadBalancingError,
                        org.omg.CORBA.CompletionStatus.COMPLETED_NO);
                }
                //catch (InterruptedException ie){}
                String ior_string = m_orb.object_to_string(object_group);
                ParsedIOR ior_parsed = new ParsedIOR(
                        (org.jacorb.orb.ORB) m_orb, ior_string);
                List profiles_list = ior_parsed.getProfiles();
                int n_profile = profiles_list.size();

                if ((lb_service_ctx.object_group_ref_version < new_version)
                        || (n_profile == 1))
                {
                    //System.out.println("    New Object group ref version: "
                    //        + new_version);
                    CDMW_DEBUG(LBInitLogger.GetLBInitLogger(), 
                               "    New Object group ref version: "  + new_version, 
                               new Throwable());        
                    // Get the most recent object group reference.
                    // Trigger LOCATION_FORWARD_PERM
                    throw new org.omg.PortableInterceptor.ForwardRequest(
                            object_group);
                }

            }
            else
            {
                //System.out
                //        .println("ERROR: Bad context id for lb_GROUP_VERSION");
                CDMW_DEBUG(LBInitLogger.GetLBInitLogger(), 
                            "    ERROR: Bad context id for lb_GROUP_VERSION"  , 
                            new Throwable());        
                throw new org.omg.CORBA.INTERNAL(
                        ExceptionMinorCodes.INTERNALLoadBalancingError,
                        org.omg.CORBA.CompletionStatus.COMPLETED_NO);
            }

        }
        catch (BAD_PARAM e)
        {
            //System.out
           //         .println("No service context LB_GROUP_VERSION for that request");

            // This is not an error while a request can come to a single
            // reference.
            // with no lb service context.
        }

        catch (SystemException e)
        {
            //System.out.println("No Check on the object group version...");
            CDMW_DEBUG(LBInitLogger.GetLBInitLogger(), 
                               "    No Check on the object group version ",  
                               new Throwable());
            e.printStackTrace();
        }

    }

    public void send_other(ServerRequestInfo ri) throws ForwardRequest
    {
        //System.out.println("    send other    ");
        CDMW_DEBUG(LBInitLogger.GetLBInitLogger(), 
                               "    send other    ",  
                               new Throwable());
        
    }
}
